from _typeshed import Incomplete, Self
from collections.abc import AsyncIterator, Awaitable, Callable, Iterable, Mapping, MutableMapping, Sequence
from datetime import datetime, timedelta
from typing import Any, ClassVar, Generic, NoReturn, Protocol, overload
from typing_extensions import Literal, TypeAlias, TypedDict

from redis import RedisError
from redis.asyncio.connection import ConnectCallbackT, Connection, ConnectionPool
from redis.asyncio.lock import Lock
from redis.asyncio.retry import Retry
from redis.client import AbstractRedis, _CommandOptions, _Key, _StrType, _Value
from redis.commands import AsyncCoreCommands, AsyncSentinelCommands, RedisModuleCommands
from redis.typing import ChannelT, EncodableT, KeyT, PatternT

PubSubHandler: TypeAlias = Callable[[dict[str, str]], Awaitable[None]]

class ResponseCallbackProtocol(Protocol):
    def __call__(self, response: Any, **kwargs): ...

class AsyncResponseCallbackProtocol(Protocol):
    async def __call__(self, response: Any, **kwargs): ...

ResponseCallbackT: TypeAlias = ResponseCallbackProtocol | AsyncResponseCallbackProtocol

class Redis(AbstractRedis, RedisModuleCommands, AsyncCoreCommands[_StrType], AsyncSentinelCommands, Generic[_StrType]):
    response_callbacks: MutableMapping[str | bytes, ResponseCallbackT]
    auto_close_connection_pool: bool
    connection_pool: Any
    single_connection_client: Any
    connection: Any
    @classmethod
    def from_url(cls, url: str, **kwargs) -> Redis[Any]: ...
    def __init__(
        self,
        *,
        host: str = ...,
        port: int = ...,
        db: str | int = ...,
        password: str | None = ...,
        socket_timeout: float | None = ...,
        socket_connect_timeout: float | None = ...,
        socket_keepalive: bool | None = ...,
        socket_keepalive_options: Mapping[int, int | bytes] | None = ...,
        connection_pool: ConnectionPool | None = ...,
        unix_socket_path: str | None = ...,
        encoding: str = ...,
        encoding_errors: str = ...,
        decode_responses: bool = ...,
        retry_on_timeout: bool = ...,
        retry_on_error: list[type[RedisError]] | None = ...,
        ssl: bool = ...,
        ssl_keyfile: str | None = ...,
        ssl_certfile: str | None = ...,
        ssl_cert_reqs: str = ...,
        ssl_ca_certs: str | None = ...,
        ssl_ca_data: str | None = ...,
        ssl_check_hostname: bool = ...,
        max_connections: int | None = ...,
        single_connection_client: bool = ...,
        health_check_interval: int = ...,
        client_name: str | None = ...,
        username: str | None = ...,
        retry: Retry | None = ...,
        auto_close_connection_pool: bool = ...,
        redis_connect_func: ConnectCallbackT | None = ...,
    ) -> None: ...
    def __await__(self): ...
    async def initialize(self: Self) -> Self: ...
    def set_response_callback(self, command: str, callback: ResponseCallbackT): ...
    def load_external_module(self, funcname, func) -> None: ...
    def pipeline(self, transaction: bool = ..., shard_hint: str | None = ...) -> Pipeline[_StrType]: ...
    async def transaction(
        self,
        func: Callable[[Pipeline[_StrType]], Any | Awaitable[Any]],
        *watches: KeyT,
        shard_hint: str | None = ...,
        value_from_callable: bool = ...,
        watch_delay: float | None = ...,
    ): ...
    def lock(
        self,
        name: KeyT,
        timeout: float | None = ...,
        sleep: float = ...,
        blocking_timeout: float | None = ...,
        lock_class: type[Lock] | None = ...,
        thread_local: bool = ...,
    ) -> Lock: ...
    def pubsub(self, **kwargs) -> PubSub: ...
    def monitor(self) -> Monitor: ...
    def client(self) -> Redis[_StrType]: ...
    async def __aenter__(self: Self) -> Self: ...
    async def __aexit__(self, exc_type, exc_value, traceback) -> None: ...
    def __del__(self, _warnings: Any = ...) -> None: ...
    async def close(self, close_connection_pool: bool | None = ...) -> None: ...
    async def execute_command(self, *args, **options): ...
    async def parse_response(self, connection: Connection, command_name: str | bytes, **options): ...

StrictRedis = Redis

class MonitorCommandInfo(TypedDict):
    time: float
    db: int
    client_address: str
    client_port: str
    client_type: str
    command: str

class Monitor:
    monitor_re: Any
    command_re: Any
    connection_pool: Any
    connection: Any
    def __init__(self, connection_pool: ConnectionPool) -> None: ...
    async def connect(self) -> None: ...
    async def __aenter__(self): ...
    async def __aexit__(self, *args) -> None: ...
    async def next_command(self) -> MonitorCommandInfo: ...
    def listen(self) -> AsyncIterator[MonitorCommandInfo]: ...

class PubSub:
    PUBLISH_MESSAGE_TYPES: ClassVar[tuple[str, ...]]
    UNSUBSCRIBE_MESSAGE_TYPES: ClassVar[tuple[str, ...]]
    HEALTH_CHECK_MESSAGE: ClassVar[str]
    connection_pool: Any
    shard_hint: str | None
    ignore_subscribe_messages: bool
    connection: Any
    encoder: Any
    health_check_response: Iterable[str | bytes]
    channels: Any
    pending_unsubscribe_channels: Any
    patterns: Any
    pending_unsubscribe_patterns: Any
    def __init__(
        self,
        connection_pool: ConnectionPool,
        shard_hint: str | None = ...,
        ignore_subscribe_messages: bool = ...,
        encoder: Any | None = ...,
    ) -> None: ...
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc_value, traceback) -> None: ...
    def __del__(self) -> None: ...
    async def reset(self) -> None: ...
    def close(self) -> Awaitable[NoReturn]: ...
    async def on_connect(self, connection: Connection): ...
    @property
    def subscribed(self) -> bool: ...
    async def execute_command(self, *args: EncodableT): ...
    async def parse_response(self, block: bool = ..., timeout: float = ...): ...
    async def check_health(self) -> None: ...
    async def psubscribe(self, *args: ChannelT, **kwargs: PubSubHandler): ...
    def punsubscribe(self, *args: ChannelT) -> Awaitable[Any]: ...
    async def subscribe(self, *args: ChannelT, **kwargs: Callable[..., Any]): ...
    def unsubscribe(self, *args) -> Awaitable[Any]: ...
    def listen(self) -> AsyncIterator[Any]: ...
    async def get_message(self, ignore_subscribe_messages: bool = ..., timeout: float = ...): ...
    def ping(self, message: Any | None = ...) -> Awaitable[Any]: ...
    async def handle_message(self, response, ignore_subscribe_messages: bool = ...): ...
    async def run(self, *, exception_handler: PSWorkerThreadExcHandlerT | None = ..., poll_timeout: float = ...) -> None: ...

class PubsubWorkerExceptionHandler(Protocol):
    def __call__(self, e: BaseException, pubsub: PubSub): ...

class AsyncPubsubWorkerExceptionHandler(Protocol):
    async def __call__(self, e: BaseException, pubsub: PubSub): ...

PSWorkerThreadExcHandlerT: TypeAlias = PubsubWorkerExceptionHandler | AsyncPubsubWorkerExceptionHandler
CommandT: TypeAlias = tuple[tuple[str | bytes, ...], Mapping[str, Any]]
CommandStackT: TypeAlias = list[CommandT]

class Pipeline(Redis[_StrType], Generic[_StrType]):
    UNWATCH_COMMANDS: ClassVar[set[str]]
    connection_pool: Any
    connection: Any
    response_callbacks: Any
    is_transaction: bool
    shard_hint: str | None
    watching: bool
    command_stack: Any
    scripts: Any
    explicit_transaction: bool
    def __init__(
        self,
        connection_pool: ConnectionPool,
        response_callbacks: MutableMapping[str | bytes, ResponseCallbackT],
        transaction: bool,
        shard_hint: str | None,
    ) -> None: ...
    async def __aenter__(self: Self) -> Self: ...
    async def __aexit__(self, exc_type, exc_value, traceback) -> None: ...
    def __await__(self): ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    async def reset(self) -> None: ...  # type: ignore[override]
    def multi(self) -> None: ...
    def execute_command(self, *args, **kwargs) -> Pipeline[_StrType] | Awaitable[Pipeline[_StrType]]: ...
    async def immediate_execute_command(self, *args, **options): ...
    def pipeline_execute_command(self, *args, **options): ...
    def raise_first_error(self, commands: CommandStackT, response: Iterable[Any]): ...
    def annotate_exception(self, exception: Exception, number: int, command: Iterable[object]) -> None: ...
    async def parse_response(self, connection: Connection, command_name: str | bytes, **options): ...
    async def load_scripts(self) -> None: ...
    async def execute(self, raise_on_error: bool = ...): ...
    async def discard(self) -> None: ...
    async def watch(self, *names: KeyT) -> bool: ...
    async def unwatch(self) -> bool: ...
    # region acl commands
    def acl_cat(self, category: str | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_deluser(self, *username: str, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_genpass(self, bits: int | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_getuser(self, username: str, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_help(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_list(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_log(self, count: int | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_log_reset(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_load(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_save(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_setuser(  # type: ignore[override]
        self,
        username: str,
        enabled: bool = ...,
        nopass: bool = ...,
        passwords: Sequence[str] | None = ...,
        hashed_passwords: Sequence[str] | None = ...,
        categories: Sequence[str] | None = ...,
        commands: Sequence[str] | None = ...,
        keys: Sequence[str] | None = ...,
        channels: Iterable[ChannelT] | None = ...,
        selectors: Iterable[tuple[str, KeyT]] | None = ...,
        reset: bool = ...,
        reset_keys: bool = ...,
        reset_passwords: bool = ...,
        **kwargs: _CommandOptions,
    ) -> Pipeline[_StrType]: ...
    def acl_users(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def acl_whoami(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    # endregion
    # region cluster commands
    def cluster(self, cluster_arg: str, *args, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def readwrite(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def readonly(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    # endregion
    # region BasicKey commands
    def append(self, key, value) -> Any: ...  # type: ignore[override]
    def bitcount(self, key: _Key, start: int | None = ..., end: int | None = ..., mode: str | None = ...) -> Any: ...  # type: ignore[override]
    def bitfield(self, key, default_overflow: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def bitop(self, operation, dest, *keys) -> Any: ...  # type: ignore[override]
    def bitpos(self, key: _Key, bit: int, start: int | None = ..., end: int | None = ..., mode: str | None = ...) -> Any: ...  # type: ignore[override]
    def copy(self, source, destination, destination_db: Incomplete | None = ..., replace: bool = ...) -> Any: ...  # type: ignore[override]
    def decr(self, name, amount: int = ...) -> Any: ...  # type: ignore[override]
    def decrby(self, name, amount: int = ...) -> Any: ...  # type: ignore[override]
    def delete(self, *names: _Key) -> Any: ...  # type: ignore[override]
    def dump(self, name: _Key) -> Any: ...  # type: ignore[override]
    def exists(self, *names: _Key) -> Any: ...  # type: ignore[override]
    def expire(  # type: ignore[override]
        self, name: _Key, time: int | timedelta, nx: bool = ..., xx: bool = ..., gt: bool = ..., lt: bool = ...
    ) -> Any: ...
    def expireat(self, name, when, nx: bool = ..., xx: bool = ..., gt: bool = ..., lt: bool = ...) -> Any: ...  # type: ignore[override]
    def get(self, name: _Key) -> Any: ...  # type: ignore[override]
    def getdel(self, name: _Key) -> Any: ...  # type: ignore[override]
    def getex(  # type: ignore[override]
        self,
        name,
        ex: Incomplete | None = ...,
        px: Incomplete | None = ...,
        exat: Incomplete | None = ...,
        pxat: Incomplete | None = ...,
        persist: bool = ...,
    ) -> Any: ...
    def getbit(self, name: _Key, offset: int) -> Any: ...  # type: ignore[override]
    def getrange(self, key, start, end) -> Any: ...  # type: ignore[override]
    def getset(self, name, value) -> Any: ...  # type: ignore[override]
    def incr(self, name: _Key, amount: int = ...) -> Any: ...  # type: ignore[override]
    def incrby(self, name: _Key, amount: int = ...) -> Any: ...  # type: ignore[override]
    def incrbyfloat(self, name: _Key, amount: float = ...) -> Any: ...  # type: ignore[override]
    def keys(self, pattern: _Key = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def lmove(  # type: ignore[override]
        self, first_list: _Key, second_list: _Key, src: Literal["LEFT", "RIGHT"] = ..., dest: Literal["LEFT", "RIGHT"] = ...
    ) -> Any: ...
    def blmove(  # type: ignore[override]
        self,
        first_list: _Key,
        second_list: _Key,
        timeout: float,
        src: Literal["LEFT", "RIGHT"] = ...,
        dest: Literal["LEFT", "RIGHT"] = ...,
    ) -> Any: ...
    def mget(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def mset(self, mapping: Mapping[_Key, _Value]) -> Any: ...  # type: ignore[override]
    def msetnx(self, mapping: Mapping[_Key, _Value]) -> Any: ...  # type: ignore[override]
    def move(self, name: _Key, db: int) -> Any: ...  # type: ignore[override]
    def persist(self, name: _Key) -> Any: ...  # type: ignore[override]
    def pexpire(  # type: ignore[override]
        self, name: _Key, time: int | timedelta, nx: bool = ..., xx: bool = ..., gt: bool = ..., lt: bool = ...
    ) -> Any: ...
    def pexpireat(  # type: ignore[override]
        self, name: _Key, when: int | datetime, nx: bool = ..., xx: bool = ..., gt: bool = ..., lt: bool = ...
    ) -> Any: ...
    def psetex(self, name, time_ms, value) -> Any: ...  # type: ignore[override]
    def pttl(self, name: _Key) -> Any: ...  # type: ignore[override]
    def hrandfield(self, key, count: Incomplete | None = ..., withvalues: bool = ...) -> Any: ...  # type: ignore[override]
    def randomkey(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def rename(self, src, dst) -> Any: ...  # type: ignore[override]
    def renamenx(self, src, dst) -> Any: ...  # type: ignore[override]
    def restore(  # type: ignore[override]
        self,
        name,
        ttl,
        value,
        replace: bool = ...,
        absttl: bool = ...,
        idletime: Incomplete | None = ...,
        frequency: Incomplete | None = ...,
    ) -> Any: ...
    def set(  # type: ignore[override]
        self,
        name: _Key,
        value: _Value,
        ex: None | int | timedelta = ...,
        px: None | int | timedelta = ...,
        nx: bool = ...,
        xx: bool = ...,
        keepttl: bool = ...,
        get: bool = ...,
        exat: Incomplete | None = ...,
        pxat: Incomplete | None = ...,
    ) -> Any: ...
    def setbit(self, name: _Key, offset: int, value: int) -> Any: ...  # type: ignore[override]
    def setex(self, name: _Key, time: int | timedelta, value: _Value) -> Any: ...  # type: ignore[override]
    def setnx(self, name: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def setrange(self, name, offset, value) -> Any: ...  # type: ignore[override]
    def stralgo(  # type: ignore[override]
        self,
        algo,
        value1,
        value2,
        specific_argument: str = ...,
        len: bool = ...,
        idx: bool = ...,
        minmatchlen: Incomplete | None = ...,
        withmatchlen: bool = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def strlen(self, name) -> Any: ...  # type: ignore[override]
    def substr(self, name, start, end: int = ...) -> Any: ...  # type: ignore[override]
    def touch(self, *args) -> Any: ...  # type: ignore[override]
    def ttl(self, name: _Key) -> Any: ...  # type: ignore[override]
    def type(self, name) -> Any: ...  # type: ignore[override]
    def unlink(self, *names: _Key) -> Any: ...  # type: ignore[override]
    # endregion
    # region hyperlog commands
    def pfadd(self, name: _Key, *values: _Value) -> Any: ...  # type: ignore[override]
    def pfcount(self, name: _Key) -> Any: ...  # type: ignore[override]
    def pfmerge(self, dest: _Key, *sources: _Key) -> Any: ...  # type: ignore[override]
    # endregion
    # region hash commands
    def hdel(self, name: _Key, *keys: _Key) -> Any: ...  # type: ignore[override]
    def hexists(self, name: _Key, key: _Key) -> Any: ...  # type: ignore[override]
    def hget(self, name: _Key, key: _Key) -> Any: ...  # type: ignore[override]
    def hgetall(self, name: _Key) -> Any: ...  # type: ignore[override]
    def hincrby(self, name: _Key, key: _Key, amount: int = ...) -> Any: ...  # type: ignore[override]
    def hincrbyfloat(self, name: _Key, key: _Key, amount: float = ...) -> Any: ...  # type: ignore[override]
    def hkeys(self, name: _Key) -> Any: ...  # type: ignore[override]
    def hlen(self, name: _Key) -> Any: ...  # type: ignore[override]
    @overload
    def hset(  # type: ignore[override]
        self, name: _Key, key: _Key, value: _Value, mapping: Mapping[_Key, _Value] | None = ..., items: Incomplete | None = ...
    ) -> Any: ...
    @overload
    def hset(self, name: _Key, key: None, value: None, mapping: Mapping[_Key, _Value], items: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    @overload
    def hset(self, name: _Key, *, mapping: Mapping[_Key, _Value], items: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def hsetnx(self, name: _Key, key: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def hmset(self, name: _Key, mapping: Mapping[_Key, _Value]) -> Any: ...  # type: ignore[override]
    def hmget(self, name: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def hvals(self, name: _Key) -> Any: ...  # type: ignore[override]
    def hstrlen(self, name, key) -> Any: ...  # type: ignore[override]
    # endregion
    # region geo commands
    def geoadd(self, name, values, nx: bool = ..., xx: bool = ..., ch: bool = ...) -> Any: ...  # type: ignore[override]
    def geodist(self, name, place1, place2, unit: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def geohash(self, name, *values) -> Any: ...  # type: ignore[override]
    def geopos(self, name, *values) -> Any: ...  # type: ignore[override]
    def georadius(  # type: ignore[override]
        self,
        name,
        longitude,
        latitude,
        radius,
        unit: Incomplete | None = ...,
        withdist: bool = ...,
        withcoord: bool = ...,
        withhash: bool = ...,
        count: Incomplete | None = ...,
        sort: Incomplete | None = ...,
        store: Incomplete | None = ...,
        store_dist: Incomplete | None = ...,
        any: bool = ...,
    ) -> Any: ...
    def georadiusbymember(  # type: ignore[override]
        self,
        name,
        member,
        radius,
        unit: Incomplete | None = ...,
        withdist: bool = ...,
        withcoord: bool = ...,
        withhash: bool = ...,
        count: Incomplete | None = ...,
        sort: Incomplete | None = ...,
        store: Incomplete | None = ...,
        store_dist: Incomplete | None = ...,
        any: bool = ...,
    ) -> Any: ...
    def geosearch(  # type: ignore[override]
        self,
        name,
        member: Incomplete | None = ...,
        longitude: Incomplete | None = ...,
        latitude: Incomplete | None = ...,
        unit: str = ...,
        radius: Incomplete | None = ...,
        width: Incomplete | None = ...,
        height: Incomplete | None = ...,
        sort: Incomplete | None = ...,
        count: Incomplete | None = ...,
        any: bool = ...,
        withcoord: bool = ...,
        withdist: bool = ...,
        withhash: bool = ...,
    ) -> Any: ...
    def geosearchstore(  # type: ignore[override]
        self,
        dest,
        name,
        member: Incomplete | None = ...,
        longitude: Incomplete | None = ...,
        latitude: Incomplete | None = ...,
        unit: str = ...,
        radius: Incomplete | None = ...,
        width: Incomplete | None = ...,
        height: Incomplete | None = ...,
        sort: Incomplete | None = ...,
        count: Incomplete | None = ...,
        any: bool = ...,
        storedist: bool = ...,
    ) -> Any: ...
    # endregion
    # region list commands
    @overload
    def blpop(self, keys: _Value | Iterable[_Value], timeout: Literal[0] | None = ...) -> Any: ...  # type: ignore[override]
    @overload
    def blpop(self, keys: _Value | Iterable[_Value], timeout: float) -> Any: ...  # type: ignore[override]
    @overload
    def brpop(self, keys: _Value | Iterable[_Value], timeout: Literal[0] | None = ...) -> Any: ...  # type: ignore[override]
    @overload
    def brpop(self, keys: _Value | Iterable[_Value], timeout: float) -> Any: ...  # type: ignore[override]
    def brpoplpush(self, src, dst, timeout: int | None = ...) -> Any: ...  # type: ignore[override]
    def lindex(self, name: _Key, index: int) -> Any: ...  # type: ignore[override]
    def linsert(  # type: ignore[override]
        self, name: _Key, where: Literal["BEFORE", "AFTER", "before", "after"], refvalue: _Value, value: _Value
    ) -> Any: ...
    def llen(self, name: _Key) -> Any: ...  # type: ignore[override]
    def lpop(self, name, count: int | None = ...) -> Any: ...  # type: ignore[override]
    def lpush(self, name: _Value, *values: _Value) -> Any: ...  # type: ignore[override]
    def lpushx(self, name, value) -> Any: ...  # type: ignore[override]
    def lrange(self, name: _Key, start: int, end: int) -> Any: ...  # type: ignore[override]
    def lrem(self, name: _Key, count: int, value: _Value) -> Any: ...  # type: ignore[override]
    def lset(self, name: _Key, index: int, value: _Value) -> Any: ...  # type: ignore[override]
    def ltrim(self, name: _Key, start: int, end: int) -> Any: ...  # type: ignore[override]
    def rpop(self, name, count: int | None = ...) -> Any: ...  # type: ignore[override]
    def rpoplpush(self, src, dst) -> Any: ...  # type: ignore[override]
    def rpush(self, name: _Value, *values: _Value) -> Any: ...  # type: ignore[override]
    def rpushx(self, name, value) -> Any: ...  # type: ignore[override]
    def lpos(self, name, value, rank: Incomplete | None = ..., count: Incomplete | None = ..., maxlen: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def sort(
        self,
        name: _Key,
        start: int | None = ...,
        num: int | None = ...,
        by: _Key | None = ...,
        get: _Key | Sequence[_Key] | None = ...,
        desc: bool = ...,
        alpha: bool = ...,
        store: None = ...,
        groups: bool = ...,
    ) -> list[_StrType]: ...
    @overload  # type: ignore[override]
    def sort(
        self,
        name: _Key,
        start: int | None = ...,
        num: int | None = ...,
        by: _Key | None = ...,
        get: _Key | Sequence[_Key] | None = ...,
        desc: bool = ...,
        alpha: bool = ...,
        *,
        store: _Key,
        groups: bool = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def sort(
        self,
        name: _Key,
        start: int | None,
        num: int | None,
        by: _Key | None,
        get: _Key | Sequence[_Key] | None,
        desc: bool,
        alpha: bool,
        store: _Key,
        groups: bool = ...,
    ) -> Any: ...
    # endregion
    # region scan commands
    def scan(  # type: ignore[override]
        self,
        cursor: int = ...,
        match: _Key | None = ...,
        count: int | None = ...,
        _type: str | None = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def sscan(self, name: _Key, cursor: int = ..., match: _Key | None = ..., count: int | None = ...) -> Any: ...  # type: ignore[override]
    def hscan(self, name: _Key, cursor: int = ..., match: _Key | None = ..., count: int | None = ...) -> Any: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def zscan(self, name: _Key, cursor: int = ..., match: _Key | None = ..., count: int | None = ...) -> Any: ...
    @overload  # type: ignore[override]
    def zscan(
        self,
        name: _Key,
        cursor: int = ...,
        match: _Key | None = ...,
        count: int | None = ...,
        *,
        score_cast_func: Callable[[_StrType], Any],
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zscan(
        self, name: _Key, cursor: int, match: _Key | None, count: int | None, score_cast_func: Callable[[_StrType], Any]
    ) -> Any: ...
    # endregion
    # region set commands
    def sadd(self, name: _Key, *values: _Value) -> Any: ...  # type: ignore[override]
    def scard(self, name: _Key) -> Any: ...  # type: ignore[override]
    def sdiff(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def sdiffstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def sinter(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def sinterstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def sismember(self, name: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def smembers(self, name: _Key) -> Any: ...  # type: ignore[override]
    def smismember(self, name, values, *args) -> Any: ...  # type: ignore[override]
    def smove(self, src: _Key, dst: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def spop(self, name: _Key, count: None = ...) -> Any: ...
    @overload  # type: ignore[override]
    def spop(self, name: _Key, count: int) -> Any: ...
    @overload  # type: ignore[override]
    def srandmember(self, name: _Key, number: None = ...) -> Any: ...
    @overload  # type: ignore[override]
    def srandmember(self, name: _Key, number: int) -> Any: ...
    def srem(self, name: _Key, *values: _Value) -> Any: ...  # type: ignore[override]
    def sunion(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    def sunionstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...  # type: ignore[override]
    # endregion
    # region stream commands
    def xack(self, name, groupname, *ids) -> Any: ...  # type: ignore[override]
    def xadd(  # type: ignore[override]
        self,
        name,
        fields,
        id: str = ...,
        maxlen=...,
        approximate: bool = ...,
        nomkstream: bool = ...,
        minid: Incomplete | None = ...,
        limit: Incomplete | None = ...,
    ) -> Any: ...
    def xautoclaim(  # type: ignore[override]
        self,
        name,
        groupname,
        consumername,
        min_idle_time,
        start_id: int = ...,
        count: Incomplete | None = ...,
        justid: bool = ...,
    ) -> Any: ...
    def xclaim(  # type: ignore[override]
        self, name, groupname, consumername, min_idle_time, message_ids, idle=..., time=..., retrycount=..., force=..., justid=...
    ) -> Any: ...
    def xdel(self, name, *ids) -> Any: ...  # type: ignore[override]
    def xgroup_create(self, name, groupname, id: str = ..., mkstream: bool = ..., entries_read: int | None = ...) -> Any: ...  # type: ignore[override]
    def xgroup_delconsumer(self, name, groupname, consumername) -> Any: ...  # type: ignore[override]
    def xgroup_destroy(self, name, groupname) -> Any: ...  # type: ignore[override]
    def xgroup_createconsumer(self, name, groupname, consumername) -> Any: ...  # type: ignore[override]
    def xgroup_setid(self, name, groupname, id, entries_read: int | None = ...) -> Any: ...  # type: ignore[override]
    def xinfo_consumers(self, name, groupname) -> Any: ...  # type: ignore[override]
    def xinfo_groups(self, name) -> Any: ...  # type: ignore[override]
    def xinfo_stream(self, name, full: bool = ...) -> Any: ...  # type: ignore[override]
    def xlen(self, name: _Key) -> Any: ...  # type: ignore[override]
    def xpending(self, name, groupname) -> Any: ...  # type: ignore[override]
    def xpending_range(  # type: ignore[override]
        self, name: _Key, groupname, min, max, count: int, consumername: Incomplete | None = ..., idle: int | None = ...
    ) -> Any: ...
    def xrange(self, name, min: str = ..., max: str = ..., count: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def xread(self, streams, count: Incomplete | None = ..., block: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def xreadgroup(  # type: ignore[override]
        self, groupname, consumername, streams, count: Incomplete | None = ..., block: Incomplete | None = ..., noack: bool = ...
    ) -> Any: ...
    def xrevrange(self, name, max: str = ..., min: str = ..., count: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def xtrim(  # type: ignore[override]
        self, name, maxlen: int | None = ..., approximate: bool = ..., minid: Incomplete | None = ..., limit: int | None = ...
    ) -> Any: ...
    # endregion
    # region sorted set commands
    def zadd(  # type: ignore[override]
        self,
        name: _Key,
        mapping: Mapping[_Key, _Value],
        nx: bool = ...,
        xx: bool = ...,
        ch: bool = ...,
        incr: bool = ...,
        gt: Incomplete | None = ...,
        lt: Incomplete | None = ...,
    ) -> Any: ...
    def zcard(self, name: _Key) -> Any: ...  # type: ignore[override]
    def zcount(self, name: _Key, min: _Value, max: _Value) -> Any: ...  # type: ignore[override]
    def zdiff(self, keys, withscores: bool = ...) -> Any: ...  # type: ignore[override]
    def zdiffstore(self, dest, keys) -> Any: ...  # type: ignore[override]
    def zincrby(self, name: _Key, amount: float, value: _Value) -> Any: ...  # type: ignore[override]
    def zinter(self, keys, aggregate: Incomplete | None = ..., withscores: bool = ...) -> Any: ...  # type: ignore[override]
    def zinterstore(self, dest: _Key, keys: Iterable[_Key], aggregate: Literal["SUM", "MIN", "MAX"] | None = ...) -> Any: ...  # type: ignore[override]
    def zlexcount(self, name: _Key, min: _Value, max: _Value) -> Any: ...  # type: ignore[override]
    def zpopmax(self, name: _Key, count: int | None = ...) -> Any: ...  # type: ignore[override]
    def zpopmin(self, name: _Key, count: int | None = ...) -> Any: ...  # type: ignore[override]
    def zrandmember(self, key, count: Incomplete | None = ..., withscores: bool = ...) -> Any: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def bzpopmax(self, keys: _Key | Iterable[_Key], timeout: Literal[0] = ...) -> Any: ...
    @overload  # type: ignore[override]
    def bzpopmax(self, keys: _Key | Iterable[_Key], timeout: float) -> Any: ...
    @overload  # type: ignore[override]
    def bzpopmin(self, keys: _Key | Iterable[_Key], timeout: Literal[0] = ...) -> Any: ...
    @overload  # type: ignore[override]
    def bzpopmin(self, keys: _Key | Iterable[_Key], timeout: float) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], Any],
        byscore: bool = ...,
        bylex: bool = ...,
        offset: int | None = ...,
        num: int | None = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], float] = ...,
        byscore: bool = ...,
        bylex: bool = ...,
        offset: int | None = ...,
        num: int | None = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], None],
        byscore: bool = ...,
        bylex: bool = ...,
        offset: int | None = ...,
        num: int | None = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], float] = ...,
        byscore: bool = ...,
        bylex: bool = ...,
        offset: int | None = ...,
        num: int | None = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool = ...,
        withscores: bool = ...,
        score_cast_func: Callable[[_StrType], Any] = ...,
        byscore: bool = ...,
        bylex: bool = ...,
        offset: int | None = ...,
        num: int | None = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrange(
        self, name: _Key, start: int, end: int, withscores: Literal[True], score_cast_func: Callable[[_StrType], None]
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrange(self, name: _Key, start: int, end: int, withscores: Literal[True]) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrange(
        self, name: _Key, start: int, end: int, withscores: bool = ..., score_cast_func: Callable[[Any], Any] = ...
    ) -> Any: ...
    def zrangestore(  # type: ignore[override]
        self,
        dest,
        name,
        start,
        end,
        byscore: bool = ...,
        bylex: bool = ...,
        desc: bool = ...,
        offset: Incomplete | None = ...,
        num: Incomplete | None = ...,
    ) -> Any: ...
    def zrangebylex(self, name: _Key, min: _Value, max: _Value, start: int | None = ..., num: int | None = ...) -> Any: ...  # type: ignore[override]
    def zrevrangebylex(self, name: _Key, max: _Value, min: _Value, start: int | None = ..., num: int | None = ...) -> Any: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def zrangebyscore(
        self,
        name: _Key,
        min: _Value,
        max: _Value,
        start: int | None = ...,
        num: int | None = ...,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], None],
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrangebyscore(
        self, name: _Key, min: _Value, max: _Value, start: int | None = ..., num: int | None = ..., *, withscores: Literal[True]
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrangebyscore(
        self,
        name: _Key,
        min: _Value,
        max: _Value,
        start: int | None = ...,
        num: int | None = ...,
        withscores: bool = ...,
        score_cast_func: Callable[[_StrType], Any] = ...,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrangebyscore(
        self,
        name: _Key,
        max: _Value,
        min: _Value,
        start: int | None = ...,
        num: int | None = ...,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], Any],
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrangebyscore(
        self, name: _Key, max: _Value, min: _Value, start: int | None = ..., num: int | None = ..., *, withscores: Literal[True]
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrangebyscore(
        self,
        name: _Key,
        max: _Value,
        min: _Value,
        start: int | None = ...,
        num: int | None = ...,
        withscores: bool = ...,
        score_cast_func: Callable[[_StrType], Any] = ...,
    ) -> Any: ...
    def zrank(self, name: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def zrem(self, name: _Key, *values: _Value) -> Any: ...  # type: ignore[override]
    def zremrangebylex(self, name: _Key, min: _Value, max: _Value) -> Any: ...  # type: ignore[override]
    def zremrangebyrank(self, name: _Key, min: int, max: int) -> Any: ...  # type: ignore[override]
    def zremrangebyscore(self, name: _Key, min: _Value, max: _Value) -> Any: ...  # type: ignore[override]
    def zrevrank(self, name: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def zscore(self, name: _Key, value: _Value) -> Any: ...  # type: ignore[override]
    def zunion(self, keys, aggregate: Incomplete | None = ..., withscores: bool = ...) -> Any: ...  # type: ignore[override]
    def zunionstore(self, dest: _Key, keys: Iterable[_Key], aggregate: Literal["SUM", "MIN", "MAX"] | None = ...) -> Any: ...  # type: ignore[override]
    def zmscore(self, key, members) -> Any: ...  # type: ignore[override]
    # endregion
    # region management commands
    def bgrewriteaof(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def bgsave(self, schedule: bool = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def role(self) -> Any: ...  # type: ignore[override]
    def client_kill(self, address: str, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_kill_filter(  # type: ignore[override]
        self,
        _id: Incomplete | None = ...,
        _type: Incomplete | None = ...,
        addr: Incomplete | None = ...,
        skipme: Incomplete | None = ...,
        laddr: Incomplete | None = ...,
        user: Incomplete | None = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def client_info(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_list(self, _type: str | None = ..., client_id: list[str] = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_getname(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_getredir(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_reply(self, reply, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_id(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_tracking_on(  # type: ignore[override]
        self,
        clientid: Incomplete | None = ...,
        prefix=...,
        bcast: bool = ...,
        optin: bool = ...,
        optout: bool = ...,
        noloop: bool = ...,
    ) -> Any: ...
    def client_tracking_off(  # type: ignore[override]
        self,
        clientid: Incomplete | None = ...,
        prefix=...,
        bcast: bool = ...,
        optin: bool = ...,
        optout: bool = ...,
        noloop: bool = ...,
    ) -> Any: ...
    def client_tracking(  # type: ignore[override]
        self,
        on: bool = ...,
        clientid: Incomplete | None = ...,
        prefix=...,
        bcast: bool = ...,
        optin: bool = ...,
        optout: bool = ...,
        noloop: bool = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def client_trackinginfo(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_setname(self, name: str, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_unblock(self, client_id, error: bool = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_pause(self, timeout, all: bool = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def client_unpause(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def command(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def command_info(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def command_count(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def config_get(self, pattern: PatternT = ..., *args: PatternT, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def config_set(self, name: KeyT, value: EncodableT, *args: KeyT | EncodableT, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def config_resetstat(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def config_rewrite(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def dbsize(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def debug_object(self, key, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def debug_segfault(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def echo(self, value: _Value, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def flushall(self, asynchronous: bool = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def flushdb(self, asynchronous: bool = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def sync(self) -> Any: ...  # type: ignore[override]
    def psync(self, replicationid, offset) -> Any: ...  # type: ignore[override]
    def swapdb(self, first, second, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def select(self, index, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def info(self, section: _Key | None = ..., *args: _Key, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def lastsave(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def lolwut(self, *version_numbers: _Value, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def migrate(  # type: ignore[override]
        self,
        host,
        port,
        keys,
        destination_db,
        timeout,
        copy: bool = ...,
        replace: bool = ...,
        auth: Incomplete | None = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def object(self, infotype, key, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_doctor(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_help(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_stats(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_malloc_stats(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_usage(self, key, samples: Incomplete | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def memory_purge(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def ping(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def quit(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def replicaof(self, *args, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def save(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def shutdown(  # type: ignore[override]
        self,
        save: bool = ...,
        nosave: bool = ...,
        now: bool = ...,
        force: bool = ...,
        abort: bool = ...,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def slaveof(self, host: Incomplete | None = ..., port: Incomplete | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def slowlog_get(self, num: Incomplete | None = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def slowlog_len(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def slowlog_reset(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def time(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def wait(self, num_replicas, timeout, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    # endregion
    # region module commands
    def module_load(self, path, *args) -> Any: ...  # type: ignore[override]
    def module_unload(self, name) -> Any: ...  # type: ignore[override]
    def module_list(self) -> Any: ...  # type: ignore[override]
    def command_getkeys(self, *args) -> Any: ...  # type: ignore[override]
    # endregion
    # region pubsub commands
    def publish(self, channel: _Key, message: _Key, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def pubsub_channels(self, pattern: _Key = ..., **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def pubsub_numpat(self, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    def pubsub_numsub(self, *args: _Key, **kwargs: _CommandOptions) -> Any: ...  # type: ignore[override]
    # endregion
    # region script commands
    def eval(self, script, numkeys, *keys_and_args) -> Any: ...  # type: ignore[override]
    def evalsha(self, sha, numkeys, *keys_and_args) -> Any: ...  # type: ignore[override]
    def script_exists(self, *args) -> Any: ...  # type: ignore[override]
    def script_debug(self, *args) -> Any: ...  # type: ignore[override]
    def script_flush(self, sync_type: Incomplete | None = ...) -> Any: ...  # type: ignore[override]
    def script_kill(self) -> Any: ...  # type: ignore[override]
    def script_load(self, script) -> Any: ...  # type: ignore[override]
    def register_script(self, script: str | _StrType) -> Any: ...  # type: ignore[override]
    # endregion



from typing import Any

def str_to_datetime_processor_factory(regexp, type_): ...
def py_fallback(): ...
def to_unicode_processor_factory(encoding, errors: Any | None = ...): ...
def to_conditional_unicode_processor_factory(encoding, errors: Any | None = ...): ...
def to_decimal_processor_factory(target_class, scale): ...



from d3dshot.capture_outputs.numpy_capture_output import NumpyCaptureOutput

# TODO: Once we can import non-types dependencies, this CaptureOutput should be float based
# See: #5768
class NumpyFloatCaptureOutput(NumpyCaptureOutput): ...



def comparator(name, first, second): ...
def current_node(): ...
def expref(expression): ...
def function_expression(name, args): ...
def field(name): ...
def filter_projection(left, right, comparator): ...
def flatten(node): ...
def identity(): ...
def index(index): ...
def index_expression(children): ...
def key_val_pair(key_name, node): ...
def literal(literal_value): ...
def multi_select_dict(nodes): ...
def multi_select_list(nodes): ...
def or_expression(left, right): ...
def and_expression(left, right): ...
def not_expression(expr): ...
def pipe(left, right): ...
def projection(left, right): ...
def subexpression(children): ...
def slice(start, end, step): ...
def value_projection(left, right): ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class TableStyleElement(Serialisable):
    tagname: str
    type: Any
    size: Any
    dxfId: Any
    def __init__(self, type: Any | None = ..., size: Any | None = ..., dxfId: Any | None = ...) -> None: ...

class TableStyle(Serialisable):
    tagname: str
    name: Any
    pivot: Any
    table: Any
    count: Any
    tableStyleElement: Any
    __elements__: Any
    def __init__(
        self,
        name: Any | None = ...,
        pivot: Any | None = ...,
        table: Any | None = ...,
        count: Any | None = ...,
        tableStyleElement=...,
    ) -> None: ...

class TableStyleList(Serialisable):
    tagname: str
    defaultTableStyle: Any
    defaultPivotStyle: Any
    tableStyle: Any
    __elements__: Any
    __attrs__: Any
    def __init__(
        self, count: Any | None = ..., defaultTableStyle: str = ..., defaultPivotStyle: str = ..., tableStyle=...
    ) -> None: ...
    @property
    def count(self): ...



from .authorization import AuthorizationEndpoint as AuthorizationEndpoint
from .introspect import IntrospectEndpoint as IntrospectEndpoint
from .metadata import MetadataEndpoint as MetadataEndpoint
from .pre_configured import (
    BackendApplicationServer as BackendApplicationServer,
    LegacyApplicationServer as LegacyApplicationServer,
    MobileApplicationServer as MobileApplicationServer,
    Server as Server,
    WebApplicationServer as WebApplicationServer,
)
from .resource import ResourceEndpoint as ResourceEndpoint
from .revocation import RevocationEndpoint as RevocationEndpoint
from .token import TokenEndpoint as TokenEndpoint



from .base import *
from .sequence import Sequence as Sequence

class MetaStrict(type):
    def __new__(cls, clsname, bases, methods): ...

class MetaSerialisable(type):
    def __new__(cls, clsname, bases, methods): ...

class Strict(metaclass=MetaStrict): ...
class _Serialiasable(metaclass=MetaSerialisable): ...



from typing import Any

from braintree.search import Search as Search
from braintree.us_bank_account import UsBankAccount as UsBankAccount
from braintree.us_bank_account_verification import UsBankAccountVerification as UsBankAccountVerification
from braintree.util import Constants as Constants

class UsBankAccountVerificationSearch:
    account_holder_name: Any
    customer_email: Any
    customer_id: Any
    id: Any
    payment_method_token: Any
    routing_number: Any
    ids: Any
    status: Any
    verification_method: Any
    created_at: Any
    account_type: Any
    account_number: Any



from builtins import (
    ChildProcessError as ChildProcessError,
    FileExistsError as FileExistsError,
    FileNotFoundError as FileNotFoundError,
    InterruptedError as InterruptedError,
    PermissionError as PermissionError,
    ProcessLookupError as ProcessLookupError,
)



from paramiko.sftp_attr import SFTPAttributes
from paramiko.util import ClosingContextManager

class SFTPHandle(ClosingContextManager):
    def __init__(self, flags: int = ...) -> None: ...
    def close(self) -> None: ...
    def read(self, offset: int, length: int) -> bytes | int: ...
    def write(self, offset: int, data: bytes) -> int: ...
    def stat(self) -> int | SFTPAttributes: ...
    def chattr(self, attr: SFTPAttributes) -> int: ...



from collections.abc import Iterator
from typing import Any

class Composable:
    def __init__(self, wrapped) -> None: ...
    def as_string(self, context) -> str: ...
    def __add__(self, other) -> Composed: ...
    def __mul__(self, n) -> Composed: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

class Composed(Composable):
    def __init__(self, seq) -> None: ...
    @property
    def seq(self) -> list[Composable]: ...
    def as_string(self, context) -> str: ...
    def __iter__(self) -> Iterator[Composable]: ...
    def __add__(self, other) -> Composed: ...
    def join(self, joiner) -> Composed: ...

class SQL(Composable):
    def __init__(self, string) -> None: ...
    @property
    def string(self) -> str: ...
    def as_string(self, context) -> str: ...
    def format(self, *args, **kwargs) -> Composed: ...
    def join(self, seq) -> Composed: ...

class Identifier(Composable):
    def __init__(self, *strings) -> None: ...
    @property
    def strings(self) -> tuple[str, ...]: ...
    @property
    def string(self) -> str: ...
    def as_string(self, context) -> str: ...

class Literal(Composable):
    @property
    def wrapped(self): ...
    def as_string(self, context) -> str: ...

class Placeholder(Composable):
    def __init__(self, name: Any | None = ...) -> None: ...
    @property
    def name(self) -> str | None: ...
    def as_string(self, context) -> str: ...

NULL: Any
DEFAULT: Any



from collections import OrderedDict
from typing import Any

from .decoder import TomlDecoder
from .encoder import TomlEncoder

class TomlOrderedDecoder(TomlDecoder[OrderedDict[str, Any]]):
    def __init__(self) -> None: ...

class TomlOrderedEncoder(TomlEncoder[OrderedDict[str, Any]]):
    def __init__(self) -> None: ...



from typing import Any
from typing_extensions import Literal

from .backends import AESKey as AESKey, ECKey as ECKey, HMACKey as HMACKey, RSAKey as RSAKey
from .backends.base import DIRKey as DIRKey, Key

def get_key(algorithm: str) -> type[Key] | None: ...
def register_key(algorithm: str, key_class: type[Key]) -> Literal[True]: ...
def construct(
    # explicitly checks for key_data as dict instance, instead of a Mapping
    key_data: str | bytes | dict[str, Any] | Key,
    algorithm: str | None = ...,
) -> Key: ...



from _typeshed import Incomplete
from typing import Any, NamedTuple

from .structure_tree import StructElem
from .syntax import Destination, PDFObject

class OutlineSection(NamedTuple):
    name: str
    level: str
    page_number: int
    dest: Destination
    struct_elem: StructElem | None = ...

class OutlineItemDictionary(PDFObject):
    title: str
    parent: Any | None
    prev: Any | None
    next: Any | None
    first: Any | None
    last: Any | None
    count: int
    dest: str | None
    struct_elem: StructElem | None
    def __init__(self, title: str, dest: str | None = ..., struct_elem: StructElem | None = ..., **kwargs) -> None: ...

class OutlineDictionary(PDFObject):
    type: str
    first: Any | None
    last: Any | None
    count: int
    def __init__(self, **kwargs) -> None: ...

def serialize_outline(sections, first_object_id: int = ..., fpdf: Any | None = ...): ...
def build_outline(sections, first_object_id, fpdf) -> tuple[Incomplete, Incomplete]: ...
def outline_as_str(outline, outline_items, fpdf) -> str: ...



from typing import Any

from openpyxl.descriptors import Float
from openpyxl.descriptors.serialisable import Serialisable

class ValueDescriptor(Float):
    expected_type: Any
    def __set__(self, instance, value) -> None: ...

class FormatObject(Serialisable):
    tagname: str
    type: Any
    val: Any
    gte: Any
    extLst: Any
    __elements__: Any
    def __init__(self, type, val: Any | None = ..., gte: Any | None = ..., extLst: Any | None = ...) -> None: ...

class RuleType(Serialisable):  # type: ignore[misc]
    cfvo: Any

class IconSet(RuleType):
    tagname: str
    iconSet: Any
    showValue: Any
    percent: Any
    reverse: Any
    __elements__: Any
    cfvo: Any
    def __init__(
        self,
        iconSet: Any | None = ...,
        showValue: Any | None = ...,
        percent: Any | None = ...,
        reverse: Any | None = ...,
        cfvo: Any | None = ...,
    ) -> None: ...

class DataBar(RuleType):
    tagname: str
    minLength: Any
    maxLength: Any
    showValue: Any
    color: Any
    __elements__: Any
    cfvo: Any
    def __init__(
        self,
        minLength: Any | None = ...,
        maxLength: Any | None = ...,
        showValue: Any | None = ...,
        cfvo: Any | None = ...,
        color: Any | None = ...,
    ) -> None: ...

class ColorScale(RuleType):
    tagname: str
    color: Any
    __elements__: Any
    cfvo: Any
    def __init__(self, cfvo: Any | None = ..., color: Any | None = ...) -> None: ...

class Rule(Serialisable):
    tagname: str
    type: Any
    dxfId: Any
    priority: Any
    stopIfTrue: Any
    aboveAverage: Any
    percent: Any
    bottom: Any
    operator: Any
    text: Any
    timePeriod: Any
    rank: Any
    stdDev: Any
    equalAverage: Any
    formula: Any
    colorScale: Any
    dataBar: Any
    iconSet: Any
    extLst: Any
    dxf: Any
    __elements__: Any
    __attrs__: Any
    def __init__(
        self,
        type,
        dxfId: Any | None = ...,
        priority: int = ...,
        stopIfTrue: Any | None = ...,
        aboveAverage: Any | None = ...,
        percent: Any | None = ...,
        bottom: Any | None = ...,
        operator: Any | None = ...,
        text: Any | None = ...,
        timePeriod: Any | None = ...,
        rank: Any | None = ...,
        stdDev: Any | None = ...,
        equalAverage: Any | None = ...,
        formula=...,
        colorScale: Any | None = ...,
        dataBar: Any | None = ...,
        iconSet: Any | None = ...,
        extLst: Any | None = ...,
        dxf: Any | None = ...,
    ) -> None: ...

def ColorScaleRule(
    start_type: Any | None = ...,
    start_value: Any | None = ...,
    start_color: Any | None = ...,
    mid_type: Any | None = ...,
    mid_value: Any | None = ...,
    mid_color: Any | None = ...,
    end_type: Any | None = ...,
    end_value: Any | None = ...,
    end_color: Any | None = ...,
): ...
def FormulaRule(
    formula: Any | None = ...,
    stopIfTrue: Any | None = ...,
    font: Any | None = ...,
    border: Any | None = ...,
    fill: Any | None = ...,
): ...
def CellIsRule(
    operator: Any | None = ...,
    formula: Any | None = ...,
    stopIfTrue: Any | None = ...,
    font: Any | None = ...,
    border: Any | None = ...,
    fill: Any | None = ...,
): ...
def IconSetRule(
    icon_style: Any | None = ...,
    type: Any | None = ...,
    values: Any | None = ...,
    showValue: Any | None = ...,
    percent: Any | None = ...,
    reverse: Any | None = ...,
): ...
def DataBarRule(
    start_type: Any | None = ...,
    start_value: Any | None = ...,
    end_type: Any | None = ...,
    end_value: Any | None = ...,
    color: Any | None = ...,
    showValue: Any | None = ...,
    minLength: Any | None = ...,
    maxLength: Any | None = ...,
): ...



import _win32typing
from win32.lib.pywintypes import error as error

def CancelWaitableTimer() -> None: ...
def CreateEvent(EventAttributes: _win32typing.PySECURITY_ATTRIBUTES, bManualReset, bInitialState, Name: str) -> int: ...
def CreateMutex(MutexAttributes: _win32typing.PySECURITY_ATTRIBUTES, InitialOwner, Name: str) -> int: ...
def CreateSemaphore(
    SemaphoreAttributes: _win32typing.PySECURITY_ATTRIBUTES, InitialCount, MaximumCount, SemaphoreName
) -> int: ...
def CreateWaitableTimer(TimerAttributes: _win32typing.PySECURITY_ATTRIBUTES, ManualReset, TimerName) -> int: ...
def MsgWaitForMultipleObjects(handlelist: list[int], bWaitAll, milliseconds, wakeMask): ...
def MsgWaitForMultipleObjectsEx(handlelist: list[int], milliseconds, wakeMask, waitFlags): ...
def OpenEvent(desiredAccess, bInheritHandle, name: str) -> int: ...
def OpenMutex(desiredAccess, bInheritHandle, name: str) -> int: ...
def OpenSemaphore(desiredAccess, bInheritHandle, name: str) -> int: ...
def OpenWaitableTimer(desiredAccess, bInheritHandle, timerName) -> int: ...
def PulseEvent(hEvent: int) -> None: ...
def ReleaseMutex(hEvent: int) -> None: ...
def ReleaseSemaphore(hEvent: int, lReleaseCount): ...
def ResetEvent(hEvent: int) -> None: ...
def SetEvent(hEvent: int) -> None: ...
def SetWaitableTimer(handle: int, dueTime, period, func, param, resume_state) -> None: ...
def WaitForMultipleObjects(handlelist: list[int], bWaitAll, milliseconds): ...
def WaitForMultipleObjectsEx(handlelist: list[int], bWaitAll, milliseconds, bAlertable): ...
def WaitForSingleObject(hHandle: int, milliseconds): ...
def WaitForSingleObjectEx(hHandle: int, milliseconds, bAlertable): ...
def WaitForInputIdle(hProcess: int, milliseconds): ...
def SignalObjectAndWait(*args, **kwargs): ...  # incomplete

EVENT_ALL_ACCESS: int
EVENT_MODIFY_STATE: int
INFINITE: int
MAXIMUM_WAIT_OBJECTS: int
QS_ALLEVENTS: int
QS_ALLINPUT: int
QS_HOTKEY: int
QS_INPUT: int
QS_KEY: int
QS_MOUSE: int
QS_MOUSEBUTTON: int
QS_MOUSEMOVE: int
QS_PAINT: int
QS_POSTMESSAGE: int
QS_SENDMESSAGE: int
QS_TIMER: int
SYNCHRONIZE: int
WAIT_ABANDONED: int
WAIT_ABANDONED_0: int
WAIT_FAILED: int
WAIT_IO_COMPLETION: int
WAIT_OBJECT_0: int
WAIT_TIMEOUT: int
UNICODE: int



from collections.abc import Iterable, Iterator, Sequence
from typing import Any

from pygments.token import _TokenType
from pygments.util import Future

class LexerMeta(type):
    def __new__(cls, name, bases, d): ...
    def analyse_text(self, text) -> None: ...  # actually defined in class Lexer
    # ClassVars of Lexer, but same situation as with StyleMeta and Style
    name: str
    aliases: Sequence[str]  # not intended mutable
    filenames: Sequence[str]
    alias_filenames: Sequence[str]
    mimetypes: Sequence[str]
    priority: int

class Lexer(metaclass=LexerMeta):
    options: Any
    stripnl: Any
    stripall: Any
    ensurenl: Any
    tabsize: Any
    encoding: Any
    filters: Any
    def __init__(self, **options) -> None: ...
    def add_filter(self, filter_, **options) -> None: ...
    def get_tokens(self, text: str, unfiltered: bool = ...) -> Iterator[tuple[_TokenType, str]]: ...
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...

class DelegatingLexer(Lexer):
    root_lexer: Any
    language_lexer: Any
    needle: Any
    def __init__(self, _root_lexer, _language_lexer, _needle=..., **options) -> None: ...
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...

class include(str): ...
class _inherit: ...

inherit: Any

class combined(tuple[Any, ...]):
    def __new__(cls, *args): ...
    def __init__(self, *args) -> None: ...

class _PseudoMatch:
    def __init__(self, start, text) -> None: ...
    def start(self, arg: Any | None = ...): ...
    def end(self, arg: Any | None = ...): ...
    def group(self, arg: Any | None = ...): ...
    def groups(self): ...
    def groupdict(self): ...

def bygroups(*args): ...

class _This: ...

this: Any

def using(_other, **kwargs): ...

class default:
    state: Any
    def __init__(self, state) -> None: ...

class words(Future):
    words: Any
    prefix: Any
    suffix: Any
    def __init__(self, words, prefix: str = ..., suffix: str = ...) -> None: ...
    def get(self): ...

class RegexLexerMeta(LexerMeta):
    def process_tokendef(cls, name, tokendefs: Any | None = ...): ...
    def get_tokendefs(cls): ...
    def __call__(cls, *args, **kwds): ...

class RegexLexer(Lexer, metaclass=RegexLexerMeta):
    flags: Any
    tokens: Any
    def get_tokens_unprocessed(self, text: str, stack: Iterable[str] = ...) -> Iterator[tuple[int, _TokenType, str]]: ...

class LexerContext:
    text: Any
    pos: Any
    end: Any
    stack: Any
    def __init__(self, text, pos, stack: Any | None = ..., end: Any | None = ...) -> None: ...

class ExtendedRegexLexer(RegexLexer):
    def get_tokens_unprocessed(  # type: ignore[override]
        self, text: str | None = ..., context: LexerContext | None = ...
    ) -> Iterator[tuple[int, _TokenType, str]]: ...

class ProfilingRegexLexerMeta(RegexLexerMeta): ...

class ProfilingRegexLexer(RegexLexer, metaclass=ProfilingRegexLexerMeta):
    def get_tokens_unprocessed(self, text: str, stack: Iterable[str] = ...) -> Iterator[tuple[int, _TokenType, str]]: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Point2D(Serialisable):
    tagname: str
    namespace: Any
    x: Any
    y: Any
    def __init__(self, x: Any | None = ..., y: Any | None = ...) -> None: ...

class PositiveSize2D(Serialisable):
    tagname: str
    namespace: Any
    cx: Any
    width: Any
    cy: Any
    height: Any
    def __init__(self, cx: Any | None = ..., cy: Any | None = ...) -> None: ...

class Transform2D(Serialisable):
    tagname: str
    namespace: Any
    rot: Any
    flipH: Any
    flipV: Any
    off: Any
    ext: Any
    chOff: Any
    chExt: Any
    __elements__: Any
    def __init__(
        self,
        rot: Any | None = ...,
        flipH: Any | None = ...,
        flipV: Any | None = ...,
        off: Any | None = ...,
        ext: Any | None = ...,
        chOff: Any | None = ...,
        chExt: Any | None = ...,
    ) -> None: ...

class GroupTransform2D(Serialisable):
    tagname: str
    namespace: Any
    rot: Any
    flipH: Any
    flipV: Any
    off: Any
    ext: Any
    chOff: Any
    chExt: Any
    __elements__: Any
    def __init__(
        self,
        rot: int = ...,
        flipH: Any | None = ...,
        flipV: Any | None = ...,
        off: Any | None = ...,
        ext: Any | None = ...,
        chOff: Any | None = ...,
        chExt: Any | None = ...,
    ) -> None: ...

class SphereCoords(Serialisable):
    tagname: str
    lat: Any
    lon: Any
    rev: Any
    def __init__(self, lat: Any | None = ..., lon: Any | None = ..., rev: Any | None = ...) -> None: ...

class Camera(Serialisable):
    tagname: str
    prst: Any
    fov: Any
    zoom: Any
    rot: Any
    def __init__(self, prst: Any | None = ..., fov: Any | None = ..., zoom: Any | None = ..., rot: Any | None = ...) -> None: ...

class LightRig(Serialisable):
    tagname: str
    rig: Any
    dir: Any
    rot: Any
    def __init__(self, rig: Any | None = ..., dir: Any | None = ..., rot: Any | None = ...) -> None: ...

class Vector3D(Serialisable):
    tagname: str
    dx: Any
    dy: Any
    dz: Any
    def __init__(self, dx: Any | None = ..., dy: Any | None = ..., dz: Any | None = ...) -> None: ...

class Point3D(Serialisable):
    tagname: str
    x: Any
    y: Any
    z: Any
    def __init__(self, x: Any | None = ..., y: Any | None = ..., z: Any | None = ...) -> None: ...

class Backdrop(Serialisable):
    anchor: Any
    norm: Any
    up: Any
    extLst: Any
    def __init__(
        self, anchor: Any | None = ..., norm: Any | None = ..., up: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...

class Scene3D(Serialisable):
    camera: Any
    lightRig: Any
    backdrop: Any
    extLst: Any
    def __init__(
        self, camera: Any | None = ..., lightRig: Any | None = ..., backdrop: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...

class Bevel(Serialisable):
    tagname: str
    w: Any
    h: Any
    prst: Any
    def __init__(self, w: Any | None = ..., h: Any | None = ..., prst: Any | None = ...) -> None: ...

class Shape3D(Serialisable):
    namespace: Any
    z: Any
    extrusionH: Any
    contourW: Any
    prstMaterial: Any
    bevelT: Any
    bevelB: Any
    extrusionClr: Any
    contourClr: Any
    extLst: Any
    def __init__(
        self,
        z: Any | None = ...,
        extrusionH: Any | None = ...,
        contourW: Any | None = ...,
        prstMaterial: Any | None = ...,
        bevelT: Any | None = ...,
        bevelB: Any | None = ...,
        extrusionClr: Any | None = ...,
        contourClr: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class Path2D(Serialisable):
    w: Any
    h: Any
    fill: Any
    stroke: Any
    extrusionOk: Any
    def __init__(
        self,
        w: Any | None = ...,
        h: Any | None = ...,
        fill: Any | None = ...,
        stroke: Any | None = ...,
        extrusionOk: Any | None = ...,
    ) -> None: ...

class Path2DList(Serialisable):
    path: Any
    def __init__(self, path: Any | None = ...) -> None: ...

class GeomRect(Serialisable):
    l: Any
    t: Any
    r: Any
    b: Any
    def __init__(self, l: Any | None = ..., t: Any | None = ..., r: Any | None = ..., b: Any | None = ...) -> None: ...

class AdjPoint2D(Serialisable):
    x: Any
    y: Any
    def __init__(self, x: Any | None = ..., y: Any | None = ...) -> None: ...

class ConnectionSite(Serialisable):
    ang: Any
    pos: Any
    def __init__(self, ang: Any | None = ..., pos: Any | None = ...) -> None: ...

class ConnectionSiteList(Serialisable):
    cxn: Any
    def __init__(self, cxn: Any | None = ...) -> None: ...

class AdjustHandleList(Serialisable): ...

class GeomGuide(Serialisable):
    name: Any
    fmla: Any
    def __init__(self, name: Any | None = ..., fmla: Any | None = ...) -> None: ...

class GeomGuideList(Serialisable):
    gd: Any
    def __init__(self, gd: Any | None = ...) -> None: ...

class CustomGeometry2D(Serialisable):
    avLst: Any
    gdLst: Any
    ahLst: Any
    cxnLst: Any
    pathLst: Any
    rect: Any
    def __init__(
        self,
        avLst: Any | None = ...,
        gdLst: Any | None = ...,
        ahLst: Any | None = ...,
        cxnLst: Any | None = ...,
        rect: Any | None = ...,
        pathLst: Any | None = ...,
    ) -> None: ...

class PresetGeometry2D(Serialisable):
    namespace: Any
    prst: Any
    avLst: Any
    def __init__(self, prst: Any | None = ..., avLst: Any | None = ...) -> None: ...

class FontReference(Serialisable):
    idx: Any
    def __init__(self, idx: Any | None = ...) -> None: ...

class StyleMatrixReference(Serialisable):
    idx: Any
    def __init__(self, idx: Any | None = ...) -> None: ...

class ShapeStyle(Serialisable):
    lnRef: Any
    fillRef: Any
    effectRef: Any
    fontRef: Any
    def __init__(
        self, lnRef: Any | None = ..., fillRef: Any | None = ..., effectRef: Any | None = ..., fontRef: Any | None = ...
    ) -> None: ...



from collections.abc import Sequence
from ctypes import _CVoidConstPLike
from typing import TypeVar
from typing_extensions import TypeAlias

from d3dshot.capture_output import CaptureOutput
from PIL import Image

_Unused: TypeAlias = object
_ImageT = TypeVar("_ImageT", bound=Image.Image)

class PILCaptureOutput(CaptureOutput):
    def __init__(self) -> None: ...
    def process(
        self,
        pointer: _CVoidConstPLike,
        pitch: int,
        size: int,
        width: int,
        height: int,
        region: tuple[int, int, int, int],
        rotation: int,
    ) -> Image.Image: ...
    def to_pil(self, frame: _ImageT) -> _ImageT: ...
    def stack(self, frames: Sequence[_ImageT], stack_dimension: _Unused) -> Sequence[_ImageT]: ...



import unittest
from datetime import datetime, timedelta
from decimal import Decimal
from fractions import Fraction

case = unittest.TestCase()

###
# Tests for assertAlmostEqual
###

case.assertAlmostEqual(1, 2.4)
case.assertAlmostEqual(2.4, 2.41)
case.assertAlmostEqual(Fraction(49, 50), Fraction(48, 50))
case.assertAlmostEqual(3.14, complex(5, 6))
case.assertAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1), delta=timedelta(hours=1))
case.assertAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1), None, "foo", timedelta(hours=1))
case.assertAlmostEqual(Decimal("1.1"), Decimal("1.11"))
case.assertAlmostEqual(2.4, 2.41, places=8)
case.assertAlmostEqual(2.4, 2.41, delta=0.02)
case.assertAlmostEqual(2.4, 2.41, None, "foo", 0.02)

case.assertAlmostEqual(2.4, 2.41, places=9, delta=0.02)  # type: ignore
case.assertAlmostEqual("foo", "bar")  # type: ignore
case.assertAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1))  # type: ignore
case.assertAlmostEqual(Decimal("0.4"), Fraction(1, 2))  # type: ignore
case.assertAlmostEqual(complex(2, 3), Decimal("0.9"))  # type: ignore

###
# Tests for assertNotAlmostEqual
###

case.assertAlmostEqual(1, 2.4)
case.assertNotAlmostEqual(Fraction(49, 50), Fraction(48, 50))
case.assertAlmostEqual(3.14, complex(5, 6))
case.assertNotAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1), delta=timedelta(hours=1))
case.assertNotAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1), None, "foo", timedelta(hours=1))

case.assertNotAlmostEqual(2.4, 2.41, places=9, delta=0.02)  # type: ignore
case.assertNotAlmostEqual("foo", "bar")  # type: ignore
case.assertNotAlmostEqual(datetime(1999, 1, 2), datetime(1999, 1, 2, microsecond=1))  # type: ignore
case.assertNotAlmostEqual(Decimal("0.4"), Fraction(1, 2))  # type: ignore
case.assertNotAlmostEqual(complex(2, 3), Decimal("0.9"))  # type: ignore

###
# Tests for assertGreater
###


class Spam:
    def __lt__(self, other: object) -> bool:
        return True


class Eggs:
    def __gt__(self, other: object) -> bool:
        return True


class Ham:
    def __lt__(self, other: "Ham") -> bool:
        if not isinstance(other, Ham):
            return NotImplemented
        return True


class Bacon:
    def __gt__(self, other: "Bacon") -> bool:
        if not isinstance(other, Bacon):
            return NotImplemented
        return True


case.assertGreater(5.8, 3)
case.assertGreater(Decimal("4.5"), Fraction(3, 2))
case.assertGreater(Fraction(3, 2), 0.9)
case.assertGreater(Eggs(), object())
case.assertGreater(object(), Spam())
case.assertGreater(Ham(), Ham())
case.assertGreater(Bacon(), Bacon())

case.assertGreater(object(), object())  # type: ignore
case.assertGreater(datetime(1999, 1, 2), 1)  # type: ignore
case.assertGreater(Spam(), Eggs())  # type: ignore
case.assertGreater(Ham(), Bacon())  # type: ignore
case.assertGreater(Bacon(), Ham())  # type: ignore



from collections.abc import Callable, Sequence
from io import TextIOWrapper
from re import Pattern
from typing import Any

HELP: str
RUNNER_PATTERN: Pattern[Any]

def match(obj_name: str) -> tuple[str, str]: ...
def resolve(module_name: str, object_name: str) -> Any: ...
def show_help(stream: TextIOWrapper, name: str, error: str | None = ...) -> None: ...
def show_exception(stream: TextIOWrapper) -> None: ...
def run(argv: Sequence[str] = ..., _serve: Callable[..., object] = ...) -> None: ...



# Referenced in: https://pyinstaller.org/en/stable/hooks.html?highlight=get_hook_config#PyInstaller.utils.hooks.get_hook_config
# Not to be imported during runtime, but is the type reference for hooks and analysis configuration

from _typeshed import Incomplete, StrPath
from collections.abc import Iterable
from typing import Any

from PyInstaller.building.datastruct import Target

class Analysis(Target):
    # https://pyinstaller.org/en/stable/hooks-config.html#hook-configuration-options
    hooksconfig: dict[str, dict[str, object]]
    def __init__(
        self,
        scripts: Iterable[StrPath],
        pathex: Incomplete | None = ...,
        binaries: Incomplete | None = ...,
        datas: Incomplete | None = ...,
        hiddenimports: Incomplete | None = ...,
        hookspath: Incomplete | None = ...,
        hooksconfig: dict[str, dict[str, Any]] | None = ...,
        excludes: Incomplete | None = ...,
        runtime_hooks: Incomplete | None = ...,
        cipher: Incomplete | None = ...,
        win_no_prefer_redirects: bool = ...,
        win_private_assemblies: bool = ...,
        noarchive: bool = ...,
        module_collection_mode: Incomplete | None = ...,
    ) -> None: ...



from .engine import (
    AsyncConnection as AsyncConnection,
    AsyncEngine as AsyncEngine,
    AsyncTransaction as AsyncTransaction,
    create_async_engine as create_async_engine,
)
from .events import AsyncConnectionEvents as AsyncConnectionEvents, AsyncSessionEvents as AsyncSessionEvents
from .result import AsyncMappingResult as AsyncMappingResult, AsyncResult as AsyncResult, AsyncScalarResult as AsyncScalarResult
from .scoping import async_scoped_session as async_scoped_session
from .session import (
    AsyncSession as AsyncSession,
    AsyncSessionTransaction as AsyncSessionTransaction,
    async_object_session as async_object_session,
    async_session as async_session,
)



from abc import ABCMeta, abstractmethod

from cryptography.hazmat.primitives.ciphers import CipherAlgorithm

class Mode(metaclass=ABCMeta):
    @property
    @abstractmethod
    def name(self) -> str: ...
    @abstractmethod
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class ModeWithAuthenticationTag(metaclass=ABCMeta):
    @property
    @abstractmethod
    def tag(self) -> bytes: ...

class ModeWithInitializationVector(metaclass=ABCMeta):
    @property
    @abstractmethod
    def initialization_vector(self) -> bytes: ...

class ModeWithNonce(metaclass=ABCMeta):
    @property
    @abstractmethod
    def nonce(self) -> bytes: ...

class ModeWithTweak(metaclass=ABCMeta):
    @property
    @abstractmethod
    def tweak(self) -> bytes: ...

class CBC(Mode, ModeWithInitializationVector):
    def __init__(self, initialization_vector: bytes) -> None: ...
    @property
    def initialization_vector(self) -> bytes: ...
    @property
    def name(self) -> str: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class CTR(Mode, ModeWithNonce):
    def __init__(self, nonce: bytes) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def nonce(self) -> bytes: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class CFB(Mode, ModeWithInitializationVector):
    def __init__(self, initialization_vector: bytes) -> None: ...
    @property
    def initialization_vector(self) -> bytes: ...
    @property
    def name(self) -> str: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class CFB8(Mode, ModeWithInitializationVector):
    def __init__(self, initialization_vector: bytes) -> None: ...
    @property
    def initialization_vector(self) -> bytes: ...
    @property
    def name(self) -> str: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class ECB(Mode):
    @property
    def name(self) -> str: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class GCM(Mode, ModeWithInitializationVector, ModeWithAuthenticationTag):
    def __init__(self, initialization_vector: bytes, tag: bytes | None = ..., min_tag_length: int | None = ...) -> None: ...
    @property
    def initialization_vector(self) -> bytes: ...
    @property
    def name(self) -> str: ...
    @property
    def tag(self) -> bytes: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class OFB(Mode, ModeWithInitializationVector):
    def __init__(self, initialization_vector: bytes) -> None: ...
    @property
    def initialization_vector(self) -> bytes: ...
    @property
    def name(self) -> str: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...

class XTS(Mode, ModeWithTweak):
    def __init__(self, tweak: bytes) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def tweak(self) -> bytes: ...
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None: ...



from collections.abc import Callable
from email import _ParamsType
from email.mime.nonmultipart import MIMENonMultipart
from email.policy import Policy

__all__ = ["MIMEApplication"]

class MIMEApplication(MIMENonMultipart):
    def __init__(
        self,
        _data: str | bytes | bytearray,
        _subtype: str = ...,
        _encoder: Callable[[MIMEApplication], object] = ...,
        *,
        policy: Policy | None = ...,
        **_params: _ParamsType,
    ) -> None: ...



from typing import Any, NoReturn

from stripe.api_resources.abstract import APIResource as APIResource
from stripe.api_resources.customer import Customer as Customer

class TaxId(APIResource):
    OBJECT_NAME: str
    def instance_url(self) -> str: ...
    @classmethod
    def retrieve(cls, id, api_key: Any | None = ..., **params) -> NoReturn: ...



from consolemenu.console_menu import MenuItem
from consolemenu.format import MenuBorderStyleType
from consolemenu.format.menu_borders import MenuBorderStyle as MenuBorderStyle, MenuBorderStyleFactory as MenuBorderStyleFactory
from consolemenu.format.menu_style import MenuStyle as MenuStyle
from consolemenu.menu_component import (
    Dimension as Dimension,
    MenuFooter as MenuFooter,
    MenuHeader as MenuHeader,
    MenuItemsSection as MenuItemsSection,
    MenuPrompt as MenuPrompt,
    MenuTextSection as MenuTextSection,
)

class MenuFormatBuilder:
    def __init__(self, max_dimension: Dimension | None = ...) -> None: ...
    def set_border_style(self, border_style: MenuBorderStyle) -> MenuFormatBuilder: ...
    def set_border_style_type(self, border_style_type: MenuBorderStyleType) -> MenuFormatBuilder: ...
    def set_border_style_factory(self, border_style_factory: MenuBorderStyleFactory) -> MenuFormatBuilder: ...
    def set_bottom_margin(self, bottom_margin: int) -> MenuFormatBuilder: ...
    def set_left_margin(self, left_margin: int) -> MenuFormatBuilder: ...
    def set_right_margin(self, right_margin: int) -> MenuFormatBuilder: ...
    def set_top_margin(self, top_margin: int) -> MenuFormatBuilder: ...
    def set_title_align(self, align: str = ...) -> MenuFormatBuilder: ...
    def set_subtitle_align(self, align: str = ...) -> MenuFormatBuilder: ...
    def set_header_left_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_header_right_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_header_bottom_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_header_top_padding(self, x: int) -> MenuFormatBuilder: ...
    def show_header_bottom_border(self, flag: bool) -> MenuFormatBuilder: ...
    def set_footer_left_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_footer_right_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_footer_bottom_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_footer_top_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_items_left_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_items_right_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_items_bottom_padding(self, x: int) -> MenuFormatBuilder: ...
    def set_items_top_padding(self, x: int) -> MenuFormatBuilder: ...
    def show_item_bottom_border(self, item_text: str, flag: bool) -> MenuFormatBuilder: ...
    def show_item_top_border(self, item_text: str, flag: bool) -> MenuFormatBuilder: ...
    def set_prologue_text_align(self, align: str = ...) -> MenuFormatBuilder: ...
    def show_prologue_top_border(self, flag: bool) -> MenuFormatBuilder: ...
    def show_prologue_bottom_border(self, flag: bool) -> MenuFormatBuilder: ...
    def set_epilogue_text_align(self, align: str = ...) -> MenuFormatBuilder: ...
    def show_epilogue_top_border(self, flag: bool) -> MenuFormatBuilder: ...
    def show_epilogue_bottom_border(self, flag: bool) -> MenuFormatBuilder: ...
    def set_prompt(self, prompt: MenuPrompt) -> MenuFormatBuilder: ...
    def clear_data(self) -> None: ...
    def format(
        self,
        title: str | None = ...,
        subtitle: str | None = ...,
        prologue_text: str | None = ...,
        epilogue_text: str | None = ...,
        items: list[MenuItem] | None = ...,
    ) -> str: ...



from _typeshed import Incomplete
from collections.abc import Sequence
from typing import NamedTuple

SOFT_HYPHEN: str
HYPHEN: str
SPACE: str
NEWLINE: str

class Fragment:
    characters: list[str]
    graphics_state: dict[str, Incomplete]
    k: float
    def __init__(self, characters: list[str] | str, graphics_state: dict[str, Incomplete], k: float) -> None: ...
    @property
    def font(self): ...
    @font.setter
    def font(self, v) -> None: ...
    @property
    def is_ttf_font(self): ...
    @property
    def font_style(self): ...
    @property
    def font_family(self): ...
    @property
    def font_size_pt(self): ...
    @property
    def font_size(self): ...
    @property
    def font_stretching(self): ...
    @property
    def char_spacing(self): ...
    @property
    def text_mode(self): ...
    @property
    def underline(self): ...
    @property
    def draw_color(self): ...
    @property
    def fill_color(self): ...
    @property
    def text_color(self): ...
    @property
    def line_width(self): ...
    @property
    def char_vpos(self): ...
    @property
    def lift(self): ...
    @property
    def string(self): ...
    def trim(self, index: int): ...
    def __eq__(self, other: Fragment) -> bool: ...  # type: ignore[override]
    def get_width(self, start: int = ..., end: int | None = ..., chars: str | None = ..., initial_cs: bool = ...): ...
    def get_character_width(self, character: str, print_sh: bool = ..., initial_cs: bool = ...): ...

class TextLine(NamedTuple):
    fragments: tuple[Incomplete, ...]
    text_width: float
    number_of_spaces: int
    justify: bool
    trailing_nl: bool = ...

class SpaceHint(NamedTuple):
    original_fragment_index: int
    original_character_index: int
    current_line_fragment_index: int
    current_line_character_index: int
    line_width: float
    number_of_spaces: int

class HyphenHint(NamedTuple):
    original_fragment_index: int
    original_character_index: int
    current_line_fragment_index: int
    current_line_character_index: int
    line_width: float
    number_of_spaces: int
    curchar: str
    curchar_width: float
    graphics_state: dict[str, Incomplete]
    k: float

class CurrentLine:
    print_sh: Incomplete
    fragments: Incomplete
    width: int
    number_of_spaces: int
    space_break_hint: Incomplete
    hyphen_break_hint: Incomplete
    def __init__(self, print_sh: bool = ...) -> None: ...
    def add_character(
        self,
        character: str,
        character_width: float,
        graphics_state: dict[str, Incomplete],
        k: float,
        original_fragment_index: int,
        original_character_index: int,
    ): ...
    def manual_break(self, justify: bool = ..., trailing_nl: bool = ...): ...
    def automatic_break_possible(self): ...
    def automatic_break(self, justify: bool): ...

class MultiLineBreak:
    styled_text_fragments: Incomplete
    justify: Incomplete
    print_sh: Incomplete
    fragment_index: int
    character_index: int
    idx_last_forced_break: Incomplete
    def __init__(self, styled_text_fragments: Sequence[Fragment], justify: bool = ..., print_sh: bool = ...) -> None: ...
    def get_line_of_given_width(self, maximum_width: float, wordsplit: bool = ...): ...



import ctypes
import sys
from typing_extensions import Literal, TypeAlias

if sys.platform == "win32":
    win32con_WM_COPYDATA: int
    def can_talk_to_agent() -> bool: ...

    ULONG_PTR: TypeAlias = ctypes.c_uint64 | ctypes.c_uint32

    class COPYDATASTRUCT(ctypes.Structure): ...

    class PageantConnection:
        def __init__(self) -> None: ...
        def send(self, data: bytes) -> None: ...
        def recv(self, n: int) -> Literal[""] | bytes: ...
        def close(self) -> None: ...



from _typeshed import Self

from stripe.api_resources.abstract.api_resource import APIResource as APIResource

class UpdateableAPIResource(APIResource):
    @classmethod
    def modify(cls: type[Self], sid: str, **params) -> Self: ...
    def save(self: Self, idempotency_key: str | None = ...) -> Self: ...



CSI: str
OSC: str
BEL: str

def code_to_chars(code: int) -> str: ...
def set_title(title: str) -> str: ...
def clear_screen(mode: int = ...) -> str: ...
def clear_line(mode: int = ...) -> str: ...

class AnsiCodes:
    def __init__(self) -> None: ...

class AnsiCursor:
    def UP(self, n: int = ...) -> str: ...
    def DOWN(self, n: int = ...) -> str: ...
    def FORWARD(self, n: int = ...) -> str: ...
    def BACK(self, n: int = ...) -> str: ...
    def POS(self, x: int = ..., y: int = ...) -> str: ...

# All attributes in the following classes are string in instances and int in the class.
# We use str since that is the common case for users.
class AnsiFore(AnsiCodes):
    BLACK: str = ...
    RED: str = ...
    GREEN: str = ...
    YELLOW: str = ...
    BLUE: str = ...
    MAGENTA: str = ...
    CYAN: str = ...
    WHITE: str = ...
    RESET: str = ...
    LIGHTBLACK_EX: str = ...
    LIGHTRED_EX: str = ...
    LIGHTGREEN_EX: str = ...
    LIGHTYELLOW_EX: str = ...
    LIGHTBLUE_EX: str = ...
    LIGHTMAGENTA_EX: str = ...
    LIGHTCYAN_EX: str = ...
    LIGHTWHITE_EX: str = ...

class AnsiBack(AnsiCodes):
    BLACK: str = ...
    RED: str = ...
    GREEN: str = ...
    YELLOW: str = ...
    BLUE: str = ...
    MAGENTA: str = ...
    CYAN: str = ...
    WHITE: str = ...
    RESET: str = ...
    LIGHTBLACK_EX: str = ...
    LIGHTRED_EX: str = ...
    LIGHTGREEN_EX: str = ...
    LIGHTYELLOW_EX: str = ...
    LIGHTBLUE_EX: str = ...
    LIGHTMAGENTA_EX: str = ...
    LIGHTCYAN_EX: str = ...
    LIGHTWHITE_EX: str = ...

class AnsiStyle(AnsiCodes):
    BRIGHT: str = ...
    DIM: str = ...
    NORMAL: str = ...
    RESET_ALL: str = ...

Fore: AnsiFore
Back: AnsiBack
Style: AnsiStyle
Cursor: AnsiCursor



from re import Pattern
from typing import Any

from ..cmd import Command

first_line_re: Pattern[str]

class build_scripts(Command):
    description: str
    user_options: Any
    boolean_options: Any
    build_dir: Any
    scripts: Any
    force: Any
    executable: Any
    outfiles: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def get_source_files(self): ...
    def run(self) -> None: ...
    def copy_scripts(self): ...



from typing import NamedTuple

from .. import exceptions

LocationParseError = exceptions.LocationParseError

url_attrs: list[str]

class _UrlBase(NamedTuple):
    auth: str | None
    fragment: str | None
    host: str | None
    path: str | None
    port: str | None
    query: str | None
    scheme: str | None

class Url(_UrlBase):
    def __new__(
        cls,
        scheme: str | None = ...,
        auth: str | None = ...,
        host: str | None = ...,
        port: str | None = ...,
        path: str | None = ...,
        query: str | None = ...,
        fragment: str | None = ...,
    ): ...
    @property
    def hostname(self) -> str | None: ...
    @property
    def request_uri(self) -> str: ...
    @property
    def netloc(self) -> str | None: ...
    @property
    def url(self) -> str: ...

def split_first(s: str, delims: str) -> tuple[str, str, str | None]: ...
def parse_url(url: str) -> Url: ...
def get_host(url: str) -> tuple[str, str | None, str | None]: ...



from .engine.events import ConnectionEvents as ConnectionEvents, DialectEvents as DialectEvents
from .pool.events import PoolEvents as PoolEvents
from .sql.base import SchemaEventTarget as SchemaEventTarget
from .sql.events import DDLEvents as DDLEvents



from typing import Any

class _PasslibRegistryProxy:
    __name__: str
    __package__: Any
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value) -> None: ...
    def __dir__(self): ...

def register_crypt_handler_path(name, path) -> None: ...
def register_crypt_handler(handler, force: bool = ..., _attr: Any | None = ...) -> None: ...
def get_crypt_handler(name, default=...): ...
def list_crypt_handlers(loaded_only: bool = ...): ...



from typing import Any
from typing_extensions import TypedDict

DJANGO_VERSION: tuple[str | int, ...] | None  # None if django is not installed
DEFAULT_ENV: str
SCHEMES: dict[str, str]

# From https://docs.djangoproject.com/en/4.0/ref/settings/#databases
class _DBConfig(TypedDict, total=False):
    ATOMIC_REQUESTS: bool
    AUTOCOMMIT: bool
    CONN_MAX_AGE: int
    DISABLE_SERVER_SIDE_CURSORS: bool
    ENGINE: str
    HOST: str
    NAME: str
    OPTIONS: dict[str, Any] | None
    PASSWORD: str
    PORT: str
    TEST: dict[str, Any]
    TIME_ZONE: str
    USER: str

def parse(url: str, engine: str | None = ..., conn_max_age: int = ..., ssl_require: bool = ...) -> _DBConfig: ...
def config(
    env: str = ..., default: str | None = ..., engine: str | None = ..., conn_max_age: int = ..., ssl_require: bool = ...
) -> _DBConfig: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Stylesheet(Serialisable):
    tagname: str
    numFmts: Any
    fonts: Any
    fills: Any
    borders: Any
    cellStyleXfs: Any
    cellXfs: Any
    cellStyles: Any
    dxfs: Any
    tableStyles: Any
    colors: Any
    extLst: Any
    __elements__: Any
    number_formats: Any
    cell_styles: Any
    alignments: Any
    protections: Any
    named_styles: Any
    def __init__(
        self,
        numFmts: Any | None = ...,
        fonts=...,
        fills=...,
        borders=...,
        cellStyleXfs: Any | None = ...,
        cellXfs: Any | None = ...,
        cellStyles: Any | None = ...,
        dxfs=...,
        tableStyles: Any | None = ...,
        colors: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node): ...
    @property
    def custom_formats(self): ...
    def to_tree(self, tagname: Any | None = ..., idx: Any | None = ..., namespace: Any | None = ...): ...

def apply_stylesheet(archive, wb): ...
def write_stylesheet(wb): ...



from typing import Union
from typing_extensions import TypeAlias

_RGB: TypeAlias = Union[tuple[int, int, int], tuple[int, int, int, int]]
_Ink: TypeAlias = str | int | _RGB
_GreyScale: TypeAlias = tuple[int, int]

def getrgb(color: _Ink) -> _RGB: ...
def getcolor(color: _Ink, mode: str) -> _RGB | _GreyScale: ...

colormap: dict[str, str]



#!/usr/bin/env python3
"""Run mypy on typeshed's stdlib and third-party stubs."""

from __future__ import annotations

import argparse
import os
import re
import sys
import tempfile
from contextlib import redirect_stderr, redirect_stdout
from dataclasses import dataclass
from io import StringIO
from itertools import product
from pathlib import Path
from typing import TYPE_CHECKING, Any, NamedTuple

if TYPE_CHECKING:
    from _typeshed import StrPath

from typing_extensions import Annotated, TypeAlias

import tomli
from utils import (
    VERSIONS_RE as VERSION_LINE_RE,
    colored,
    get_gitignore_spec,
    get_recursive_requirements,
    print_error,
    print_success_msg,
    spec_matches_path,
    strip_comments,
)

SUPPORTED_VERSIONS = ["3.11", "3.10", "3.9", "3.8", "3.7"]
SUPPORTED_PLATFORMS = ("linux", "win32", "darwin")
DIRECTORIES_TO_TEST = [Path("stdlib"), Path("stubs")]

ReturnCode: TypeAlias = int
VersionString: TypeAlias = Annotated[str, "Must be one of the entries in SUPPORTED_VERSIONS"]
VersionTuple: TypeAlias = tuple[int, int]
Platform: TypeAlias = Annotated[str, "Must be one of the entries in SUPPORTED_PLATFORMS"]


class CommandLineArgs(argparse.Namespace):
    verbose: int
    filter: list[Path]
    exclude: list[Path] | None
    python_version: list[VersionString] | None
    platform: list[Platform] | None


def valid_path(cmd_arg: str) -> Path:
    """Helper function for argument-parsing"""
    path = Path(cmd_arg)
    if not path.exists():
        raise argparse.ArgumentTypeError(f'"{path}" does not exist in typeshed!')
    if not (path in DIRECTORIES_TO_TEST or any(directory in path.parents for directory in DIRECTORIES_TO_TEST)):
        raise argparse.ArgumentTypeError('mypy_test.py only tests the stubs found in the "stdlib" and "stubs" directories')
    return path


parser = argparse.ArgumentParser(
    description="Typecheck typeshed's stubs with mypy. Patterns are unanchored regexps on the full path."
)
parser.add_argument(
    "filter",
    type=valid_path,
    nargs="*",
    help='Test these files and directories (defaults to all files in the "stdlib" and "stubs" directories)',
)
parser.add_argument("-x", "--exclude", type=valid_path, nargs="*", help="Exclude these files and directories")
parser.add_argument("-v", "--verbose", action="count", default=0, help="More output")
parser.add_argument(
    "-p",
    "--python-version",
    type=str,
    choices=SUPPORTED_VERSIONS,
    nargs="*",
    action="extend",
    help="These versions only (major[.minor])",
)
parser.add_argument(
    "--platform",
    choices=SUPPORTED_PLATFORMS,
    nargs="*",
    action="extend",
    help="Run mypy for certain OS platforms (defaults to sys.platform only)",
)


@dataclass
class TestConfig:
    """Configuration settings for a single run of the `test_typeshed` function."""

    verbose: int
    filter: list[Path]
    exclude: list[Path]
    version: VersionString
    platform: Platform


def log(args: TestConfig, *varargs: object) -> None:
    if args.verbose >= 2:
        print(*varargs)


def match(path: Path, args: TestConfig) -> bool:
    for excluded_path in args.exclude:
        if path == excluded_path:
            log(args, path, "explicitly excluded")
            return False
        if excluded_path in path.parents:
            log(args, path, f'is in an explicitly excluded directory "{excluded_path}"')
            return False
    for included_path in args.filter:
        if path == included_path:
            log(args, path, "was explicitly included")
            return True
        if included_path in path.parents:
            log(args, path, f'is in an explicitly included directory "{included_path}"')
            return True
    log_msg = (
        f'is implicitly excluded: was not in any of the directories or paths specified on the command line: "{args.filter!r}"'
    )
    log(args, path, log_msg)
    return False


def parse_versions(fname: StrPath) -> dict[str, tuple[VersionTuple, VersionTuple]]:
    result = {}
    with open(fname, encoding="UTF-8") as f:
        for line in f:
            line = strip_comments(line)
            if line == "":
                continue
            m = VERSION_LINE_RE.match(line)
            assert m, f"invalid VERSIONS line: {line}"
            mod: str = m.group(1)
            min_version = parse_version(m.group(2))
            max_version = parse_version(m.group(3)) if m.group(3) else (99, 99)
            result[mod] = min_version, max_version
    return result


_VERSION_RE = re.compile(r"^([23])\.(\d+)$")


def parse_version(v_str: str) -> tuple[int, int]:
    m = _VERSION_RE.match(v_str)
    assert m, f"invalid version: {v_str}"
    return int(m.group(1)), int(m.group(2))


def add_files(files: list[Path], module: Path, args: TestConfig) -> None:
    """Add all files in package or module represented by 'name' located in 'root'."""
    if module.is_file() and module.suffix == ".pyi":
        if match(module, args):
            files.append(module)
    else:
        files.extend(sorted(file for file in module.rglob("*.pyi") if match(file, args)))


class MypyDistConf(NamedTuple):
    module_name: str
    values: dict[str, dict[str, Any]]


# The configuration section in the metadata file looks like the following, with multiple module sections possible
# [mypy-tests]
# [mypy-tests.yaml]
# module_name = "yaml"
# [mypy-tests.yaml.values]
# disallow_incomplete_defs = true
# disallow_untyped_defs = true


def add_configuration(configurations: list[MypyDistConf], distribution: str) -> None:
    with Path("stubs", distribution, "METADATA.toml").open("rb") as f:
        data = tomli.load(f)

    mypy_tests_conf = data.get("mypy-tests")
    if not mypy_tests_conf:
        return

    assert isinstance(mypy_tests_conf, dict), "mypy-tests should be a section"
    for section_name, mypy_section in mypy_tests_conf.items():
        assert isinstance(mypy_section, dict), f"{section_name} should be a section"
        module_name = mypy_section.get("module_name")

        assert module_name is not None, f"{section_name} should have a module_name key"
        assert isinstance(module_name, str), f"{section_name} should be a key-value pair"

        values = mypy_section.get("values")
        assert values is not None, f"{section_name} should have a values section"
        assert isinstance(values, dict), "values should be a section"

        configurations.append(MypyDistConf(module_name, values.copy()))


def run_mypy(args: TestConfig, configurations: list[MypyDistConf], files: list[Path]) -> ReturnCode:
    try:
        from mypy.api import run as mypy_run
    except ImportError:
        print_error("Cannot import mypy. Did you install it?")
        sys.exit(1)

    with tempfile.NamedTemporaryFile("w+") as temp:
        temp.write("[mypy]\n")
        for dist_conf in configurations:
            temp.write(f"[mypy-{dist_conf.module_name}]\n")
            for k, v in dist_conf.values.items():
                temp.write(f"{k} = {v}\n")
        temp.flush()

        flags = get_mypy_flags(args, temp.name)
        mypy_args = [*flags, *map(str, files)]
        if args.verbose:
            print("running mypy", " ".join(mypy_args))
        stdout_redirect, stderr_redirect = StringIO(), StringIO()
        with redirect_stdout(stdout_redirect), redirect_stderr(stderr_redirect):
            returned_stdout, returned_stderr, exit_code = mypy_run(mypy_args)

        if exit_code:
            print_error("failure\n")
            captured_stdout = stdout_redirect.getvalue()
            captured_stderr = stderr_redirect.getvalue()
            if returned_stderr:
                print_error(returned_stderr)
            if captured_stderr:
                print_error(captured_stderr)
            if returned_stdout:
                print_error(returned_stdout)
            if captured_stdout:
                print_error(captured_stdout, end="")
        else:
            print_success_msg()
        return exit_code


def get_mypy_flags(args: TestConfig, temp_name: str) -> list[str]:
    return [
        "--python-version",
        args.version,
        "--show-traceback",
        "--warn-incomplete-stub",
        "--show-error-codes",
        "--no-error-summary",
        "--platform",
        args.platform,
        "--no-site-packages",
        "--custom-typeshed-dir",
        str(Path(__file__).parent.parent),
        "--no-implicit-optional",
        "--disallow-untyped-decorators",
        "--disallow-any-generics",
        "--strict-equality",
        "--enable-error-code",
        "ignore-without-code",
        "--config-file",
        temp_name,
    ]


def add_third_party_files(
    distribution: str, files: list[Path], args: TestConfig, configurations: list[MypyDistConf], seen_dists: set[str]
) -> None:
    if distribution in seen_dists:
        return
    seen_dists.add(distribution)

    stubs_dir = Path("stubs")
    dependencies = get_recursive_requirements(distribution)

    for dependency in dependencies:
        if dependency in seen_dists:
            continue
        seen_dists.add(dependency)
        files_to_add = sorted((stubs_dir / dependency).rglob("*.pyi"))
        files.extend(files_to_add)
        for file in files_to_add:
            log(args, file, f"included as a dependency of {distribution!r}")

    root = stubs_dir / distribution
    for name in os.listdir(root):
        if name.startswith("."):
            continue
        add_files(files, (root / name), args)
        add_configuration(configurations, distribution)


class TestResults(NamedTuple):
    exit_code: int
    files_checked: int


def test_third_party_distribution(distribution: str, args: TestConfig) -> TestResults:
    """Test the stubs of a third-party distribution.

    Return a tuple, where the first element indicates mypy's return code
    and the second element is the number of checked files.
    """

    files: list[Path] = []
    configurations: list[MypyDistConf] = []
    seen_dists: set[str] = set()
    add_third_party_files(distribution, files, args, configurations, seen_dists)

    if not files and args.filter:
        return TestResults(0, 0)

    print(f"testing {distribution} ({len(files)} files)... ", end="")

    if not files:
        print_error("no files found")
        sys.exit(1)

    code = run_mypy(args, configurations, files)
    return TestResults(code, len(files))


def test_stdlib(code: int, args: TestConfig) -> TestResults:
    files: list[Path] = []
    stdlib = Path("stdlib")
    supported_versions = parse_versions(stdlib / "VERSIONS")
    for name in os.listdir(stdlib):
        if name == "VERSIONS" or name.startswith("."):
            continue
        module = Path(name).stem
        module_min_version, module_max_version = supported_versions[module]
        if module_min_version <= tuple(map(int, args.version.split("."))) <= module_max_version:
            add_files(files, (stdlib / name), args)

    if files:
        print(f"Testing stdlib ({len(files)} files)...")
        print("Running mypy " + " ".join(get_mypy_flags(args, "/tmp/...")))
        this_code = run_mypy(args, [], files)
        code = max(code, this_code)

    return TestResults(code, len(files))


def test_third_party_stubs(code: int, args: TestConfig) -> TestResults:
    print("Testing third-party packages...")
    print("Running mypy " + " ".join(get_mypy_flags(args, "/tmp/...")))
    files_checked = 0
    gitignore_spec = get_gitignore_spec()

    for distribution in sorted(os.listdir("stubs")):
        distribution_path = Path("stubs", distribution)

        if spec_matches_path(gitignore_spec, distribution_path):
            continue

        if (
            distribution_path in args.filter
            or Path("stubs") in args.filter
            or any(distribution_path in path.parents for path in args.filter)
        ):
            this_code, checked = test_third_party_distribution(distribution, args)
            code = max(code, this_code)
            files_checked += checked

    return TestResults(code, files_checked)


def test_typeshed(code: int, args: TestConfig) -> TestResults:
    print(f"*** Testing Python {args.version} on {args.platform}")
    files_checked_this_version = 0
    stdlib_dir, stubs_dir = Path("stdlib"), Path("stubs")
    if stdlib_dir in args.filter or any(stdlib_dir in path.parents for path in args.filter):
        code, stdlib_files_checked = test_stdlib(code, args)
        files_checked_this_version += stdlib_files_checked
        print()

    if stubs_dir in args.filter or any(stubs_dir in path.parents for path in args.filter):
        code, third_party_files_checked = test_third_party_stubs(code, args)
        files_checked_this_version += third_party_files_checked
        print()

    return TestResults(code, files_checked_this_version)


def main() -> None:
    args = parser.parse_args(namespace=CommandLineArgs())
    versions = args.python_version or SUPPORTED_VERSIONS
    platforms = args.platform or [sys.platform]
    filter = args.filter or DIRECTORIES_TO_TEST
    exclude = args.exclude or []
    code = 0
    total_files_checked = 0
    for version, platform in product(versions, platforms):
        config = TestConfig(args.verbose, filter, exclude, version, platform)
        code, files_checked_this_version = test_typeshed(code, args=config)
        total_files_checked += files_checked_this_version
    if code:
        print_error(f"--- exit status {code}, {total_files_checked} files checked ---")
        sys.exit(code)
    if not total_files_checked:
        print_error("--- nothing to do; exit 1 ---")
        sys.exit(1)
    print(colored(f"--- success, {total_files_checked} files checked ---", "green"))


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print_error("\n\n!!!\nTest aborted due to KeyboardInterrupt\n!!!")
        sys.exit(1)



from collections.abc import Callable

from consolemenu.console_menu import ConsoleMenu
from consolemenu.items import MenuItem as MenuItem

class SubmenuItem(MenuItem):
    submenu: ConsoleMenu
    def __init__(
        self, text: str | Callable[[], str], submenu: ConsoleMenu, menu: ConsoleMenu | None = ..., should_exit: bool = ...
    ) -> None: ...
    menu: ConsoleMenu
    def set_menu(self, menu: ConsoleMenu) -> None: ...
    def set_up(self) -> None: ...
    def action(self) -> None: ...
    def clean_up(self) -> None: ...
    def get_return(self) -> object: ...
    def get_submenu(self) -> ConsoleMenu: ...



import _random
import sys
from _typeshed import SupportsLenAndGetItem
from collections.abc import Callable, Iterable, MutableSequence, Sequence, Set as AbstractSet
from fractions import Fraction
from typing import Any, ClassVar, NoReturn, TypeVar

__all__ = [
    "Random",
    "seed",
    "random",
    "uniform",
    "randint",
    "choice",
    "sample",
    "randrange",
    "shuffle",
    "normalvariate",
    "lognormvariate",
    "expovariate",
    "vonmisesvariate",
    "gammavariate",
    "triangular",
    "gauss",
    "betavariate",
    "paretovariate",
    "weibullvariate",
    "getstate",
    "setstate",
    "getrandbits",
    "choices",
    "SystemRandom",
]

if sys.version_info >= (3, 9):
    __all__ += ["randbytes"]

_T = TypeVar("_T")

class Random(_random.Random):
    VERSION: ClassVar[int]
    def __init__(self, x: Any = ...) -> None: ...
    # Using other `seed` types is deprecated since 3.9 and removed in 3.11
    # Ignore Y041, since random.seed doesn't treat int like a float subtype. Having an explicit
    # int better documents conventional usage of random.seed.
    if sys.version_info >= (3, 9):
        def seed(self, a: int | float | str | bytes | bytearray | None = ..., version: int = ...) -> None: ...  # type: ignore[override]  # noqa: Y041
    else:
        def seed(self, a: Any = ..., version: int = ...) -> None: ...

    def getstate(self) -> tuple[Any, ...]: ...
    def setstate(self, state: tuple[Any, ...]) -> None: ...
    def randrange(self, start: int, stop: int | None = ..., step: int = ...) -> int: ...
    def randint(self, a: int, b: int) -> int: ...
    if sys.version_info >= (3, 9):
        def randbytes(self, n: int) -> bytes: ...

    def choice(self, seq: SupportsLenAndGetItem[_T]) -> _T: ...
    def choices(
        self,
        population: SupportsLenAndGetItem[_T],
        weights: Sequence[float | Fraction] | None = ...,
        *,
        cum_weights: Sequence[float | Fraction] | None = ...,
        k: int = ...,
    ) -> list[_T]: ...
    if sys.version_info >= (3, 11):
        def shuffle(self, x: MutableSequence[Any]) -> None: ...
    else:
        def shuffle(self, x: MutableSequence[Any], random: Callable[[], float] | None = ...) -> None: ...
    if sys.version_info >= (3, 11):
        def sample(self, population: Sequence[_T], k: int, *, counts: Iterable[int] | None = ...) -> list[_T]: ...
    elif sys.version_info >= (3, 9):
        def sample(
            self, population: Sequence[_T] | AbstractSet[_T], k: int, *, counts: Iterable[int] | None = ...
        ) -> list[_T]: ...
    else:
        def sample(self, population: Sequence[_T] | AbstractSet[_T], k: int) -> list[_T]: ...

    def uniform(self, a: float, b: float) -> float: ...
    def triangular(self, low: float = ..., high: float = ..., mode: float | None = ...) -> float: ...
    def betavariate(self, alpha: float, beta: float) -> float: ...
    def expovariate(self, lambd: float) -> float: ...
    def gammavariate(self, alpha: float, beta: float) -> float: ...
    if sys.version_info >= (3, 11):
        def gauss(self, mu: float = ..., sigma: float = ...) -> float: ...
        def normalvariate(self, mu: float = ..., sigma: float = ...) -> float: ...
    else:
        def gauss(self, mu: float, sigma: float) -> float: ...
        def normalvariate(self, mu: float, sigma: float) -> float: ...

    def lognormvariate(self, mu: float, sigma: float) -> float: ...
    def vonmisesvariate(self, mu: float, kappa: float) -> float: ...
    def paretovariate(self, alpha: float) -> float: ...
    def weibullvariate(self, alpha: float, beta: float) -> float: ...

# SystemRandom is not implemented for all OS's; good on Windows & Linux
class SystemRandom(Random):
    def getrandbits(self, k: int) -> int: ...  # k can be passed by keyword
    def getstate(self, *args: Any, **kwds: Any) -> NoReturn: ...
    def setstate(self, *args: Any, **kwds: Any) -> NoReturn: ...

# ----- random function stubs -----

_inst: Random = ...
seed = _inst.seed
random = _inst.random
uniform = _inst.uniform
triangular = _inst.triangular
randint = _inst.randint
choice = _inst.choice
randrange = _inst.randrange
sample = _inst.sample
shuffle = _inst.shuffle
choices = _inst.choices
normalvariate = _inst.normalvariate
lognormvariate = _inst.lognormvariate
expovariate = _inst.expovariate
vonmisesvariate = _inst.vonmisesvariate
gammavariate = _inst.gammavariate
gauss = _inst.gauss
betavariate = _inst.betavariate
paretovariate = _inst.paretovariate
weibullvariate = _inst.weibullvariate
getstate = _inst.getstate
setstate = _inst.setstate
getrandbits = _inst.getrandbits
if sys.version_info >= (3, 9):
    randbytes = _inst.randbytes



from typing_extensions import LiteralString

from PyInstaller import compat as compat

__all__ = ("HOMEPATH", "PLATFORM", "__version__", "DEFAULT_DISTPATH", "DEFAULT_SPECPATH", "DEFAULT_WORKPATH")
__version__: str
HOMEPATH: str
DEFAULT_SPECPATH: str
DEFAULT_DISTPATH: str
DEFAULT_WORKPATH: str
PLATFORM: LiteralString



from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class PixarImageFile(ImageFile):
    format: ClassVar[Literal["PIXAR"]]
    format_description: ClassVar[str]



from _typeshed import Self
from collections.abc import Callable
from hashlib import _Hash
from logging import Logger, LogRecord
from types import TracebackType
from typing import IO, AnyStr, Protocol, TypeVar

from paramiko.config import SSHConfig, SSHConfigDict
from paramiko.hostkeys import HostKeys

class SupportsClose(Protocol):
    def close(self) -> None: ...

_T = TypeVar("_T")

def inflate_long(s: bytes, always_positive: bool = ...) -> int: ...

deflate_zero: int
deflate_ff: int

def deflate_long(n: int, add_sign_padding: bool = ...) -> bytes: ...
def format_binary(data: bytes, prefix: str = ...) -> list[str]: ...
def format_binary_line(data: bytes) -> str: ...
def safe_string(s: bytes) -> bytes: ...
def bit_length(n: int) -> int: ...
def tb_strings() -> list[str]: ...
def generate_key_bytes(hash_alg: type[_Hash], salt: bytes, key: bytes | str, nbytes: int) -> bytes: ...
def load_host_keys(filename: str) -> HostKeys: ...
def parse_ssh_config(file_obj: IO[str]) -> SSHConfig: ...
def lookup_ssh_host_config(hostname: str, config: SSHConfig) -> SSHConfigDict: ...
def mod_inverse(x: int, m: int) -> int: ...
def get_thread_id() -> int: ...
def log_to_file(filename: str, level: int = ...) -> None: ...

class PFilter:
    def filter(self, record: LogRecord) -> bool: ...

def get_logger(name: str) -> Logger: ...
def retry_on_signal(function: Callable[[], _T]) -> _T: ...
def constant_time_bytes_eq(a: AnyStr, b: AnyStr) -> bool: ...

class ClosingContextManager:
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...

def clamp_value(minimum: int, val: int, maximum: int) -> int: ...



from typing import Any, Tuple, Union


# The following should pass without error (see #6661):
class Diagnostic:
    def __reduce__(self) -> Union[str, Tuple[Any, ...]]:
        res = super().__reduce__()
        if isinstance(res, tuple) and len(res) >= 3:
            res[2]["_info"] = 42

        return res



from typing import Any

class AttrDef:
    name: Any
    key: Any
    validate: Any
    pre_query: Any
    post_query: Any
    default: Any
    dereference_dn: Any
    description: Any
    mandatory: Any
    single_value: Any
    oid_info: Any
    other_names: Any
    def __init__(
        self,
        name,
        key: Any | None = ...,
        validate: Any | None = ...,
        pre_query: Any | None = ...,
        post_query: Any | None = ...,
        default=...,
        dereference_dn: Any | None = ...,
        description: Any | None = ...,
        mandatory: bool = ...,
        single_value: Any | None = ...,
        alias: Any | None = ...,
    ) -> None: ...
    def __eq__(self, other): ...
    def __lt__(self, other): ...
    def __hash__(self): ...
    def __setattr__(self, key, value) -> None: ...



from _typeshed import Incomplete

from win32.lib.pywintypes import error as error

def AddCounter(hQuery, path: str, userData: int = ...): ...
def AddEnglishCounter(hQuery, path: str, userData: int = ...): ...
def RemoveCounter(handle) -> None: ...
def EnumObjectItems(DataSource: str, machine: str, _object: str, detailLevel, flags=...): ...
def EnumObjects(DataSource: str, machine: str, detailLevel, refresh: int = ...): ...
def OpenQuery(DataSource: Incomplete | None = ..., userData: int = ...): ...
def CloseQuery(handle) -> None: ...
def MakeCounterPath(
    elements: tuple[Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete], flags=...
) -> None: ...
def GetCounterInfo(handle, bRetrieveExplainText) -> None: ...
def GetFormattedCounterValue(handle, _format) -> tuple[Incomplete, Incomplete]: ...
def CollectQueryData(hQuery) -> None: ...
def ValidatePath(path: str): ...
def ExpandCounterPath(wildCardPath: str) -> tuple[Incomplete, Incomplete]: ...
def ParseCounterPath(path: str, flags=...) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete]: ...
def ParseInstanceName(instanceName: str) -> tuple[Incomplete, Incomplete, Incomplete]: ...
def SetCounterScaleFactor(hCounter, factor) -> None: ...
def BrowseCounters(
    Flags: tuple[Incomplete, ...],
    hWndOwner: int,
    CallBack1,
    CallBack2,
    DialogBoxCaption: str | None = ...,
    InitialPath: Incomplete | None = ...,
    DataSource: Incomplete | None = ...,
    ReturnMultiple: bool = ...,
    CallBackArg: Incomplete | None = ...,
) -> str: ...
def ConnectMachine(machineName: str) -> str: ...
def LookupPerfIndexByName(machineName: str, instanceName: str): ...
def LookupPerfNameByIndex(machineName: str, index) -> str: ...
def GetFormattedCounterArray(*args, **kwargs): ...  # incomplete

PDH_FMT_1000: int
PDH_FMT_ANSI: int
PDH_FMT_DOUBLE: int
PDH_FMT_LARGE: int
PDH_FMT_LONG: int
PDH_FMT_NODATA: int
PDH_FMT_NOSCALE: int
PDH_FMT_RAW: int
PDH_FMT_UNICODE: int
PDH_MAX_SCALE: int
PDH_MIN_SCALE: int
PDH_PATH_WBEM_INPUT: int
PDH_PATH_WBEM_RESULT: int
PDH_VERSION: int
PERF_DETAIL_ADVANCED: int
PERF_DETAIL_EXPERT: int
PERF_DETAIL_NOVICE: int
PERF_DETAIL_WIZARD: int

class counter_status_error(Exception): ...

PDH_FMT_NOCAP100: int



from typing import Any

from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension

PIPE_NONE: int
PIPE_LEFT: int
PIPE_RIGHT: int

class TableProcessor(BlockProcessor):
    RE_CODE_PIPES: Any
    RE_END_BORDER: Any
    border: bool = ...
    separator: str = ...
    def __init__(self, parser, config) -> None: ...

class TableExtension(Extension): ...

def makeExtension(**kwargs): ...



from typing import Any

from braintree.apple_pay_options import ApplePayOptions as ApplePayOptions
from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.unexpected_error import UnexpectedError as UnexpectedError
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class ApplePayGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def register_domain(self, domain): ...
    def unregister_domain(self, domain): ...
    def registered_domains(self): ...



from stripe.api_resources import (
    billing_portal as billing_portal,
    checkout as checkout,
    identity as identity,
    issuing as issuing,
    radar as radar,
    reporting as reporting,
    sigma as sigma,
    terminal as terminal,
)
from stripe.api_resources.account import Account as Account
from stripe.api_resources.account_link import AccountLink as AccountLink
from stripe.api_resources.alipay_account import AlipayAccount as AlipayAccount
from stripe.api_resources.apple_pay_domain import ApplePayDomain as ApplePayDomain
from stripe.api_resources.application_fee import ApplicationFee as ApplicationFee
from stripe.api_resources.application_fee_refund import ApplicationFeeRefund as ApplicationFeeRefund
from stripe.api_resources.balance import Balance as Balance
from stripe.api_resources.balance_transaction import BalanceTransaction as BalanceTransaction
from stripe.api_resources.bank_account import BankAccount as BankAccount
from stripe.api_resources.bitcoin_receiver import BitcoinReceiver as BitcoinReceiver
from stripe.api_resources.bitcoin_transaction import BitcoinTransaction as BitcoinTransaction
from stripe.api_resources.capability import Capability as Capability
from stripe.api_resources.card import Card as Card
from stripe.api_resources.charge import Charge as Charge
from stripe.api_resources.country_spec import CountrySpec as CountrySpec
from stripe.api_resources.coupon import Coupon as Coupon
from stripe.api_resources.credit_note import CreditNote as CreditNote
from stripe.api_resources.credit_note_line_item import CreditNoteLineItem as CreditNoteLineItem
from stripe.api_resources.customer import Customer as Customer
from stripe.api_resources.customer_balance_transaction import CustomerBalanceTransaction as CustomerBalanceTransaction
from stripe.api_resources.dispute import Dispute as Dispute
from stripe.api_resources.ephemeral_key import EphemeralKey as EphemeralKey
from stripe.api_resources.error_object import ErrorObject as ErrorObject, OAuthErrorObject as OAuthErrorObject
from stripe.api_resources.event import Event as Event
from stripe.api_resources.exchange_rate import ExchangeRate as ExchangeRate
from stripe.api_resources.file import File as File, FileUpload as FileUpload
from stripe.api_resources.file_link import FileLink as FileLink
from stripe.api_resources.invoice import Invoice as Invoice
from stripe.api_resources.invoice_item import InvoiceItem as InvoiceItem
from stripe.api_resources.invoice_line_item import InvoiceLineItem as InvoiceLineItem
from stripe.api_resources.issuer_fraud_record import IssuerFraudRecord as IssuerFraudRecord
from stripe.api_resources.line_item import LineItem as LineItem
from stripe.api_resources.list_object import ListObject as ListObject
from stripe.api_resources.login_link import LoginLink as LoginLink
from stripe.api_resources.mandate import Mandate as Mandate
from stripe.api_resources.order import Order as Order
from stripe.api_resources.payment_intent import PaymentIntent as PaymentIntent
from stripe.api_resources.payment_method import PaymentMethod as PaymentMethod
from stripe.api_resources.payout import Payout as Payout
from stripe.api_resources.person import Person as Person
from stripe.api_resources.plan import Plan as Plan
from stripe.api_resources.price import Price as Price
from stripe.api_resources.product import Product as Product
from stripe.api_resources.promotion_code import PromotionCode as PromotionCode
from stripe.api_resources.quote import Quote as Quote
from stripe.api_resources.recipient import Recipient as Recipient
from stripe.api_resources.recipient_transfer import RecipientTransfer as RecipientTransfer
from stripe.api_resources.refund import Refund as Refund
from stripe.api_resources.reversal import Reversal as Reversal
from stripe.api_resources.review import Review as Review
from stripe.api_resources.search_result_object import SearchResultObject as SearchResultObject
from stripe.api_resources.setup_attempt import SetupAttempt as SetupAttempt
from stripe.api_resources.setup_intent import SetupIntent as SetupIntent
from stripe.api_resources.sku import SKU as SKU
from stripe.api_resources.source import Source as Source
from stripe.api_resources.source_transaction import SourceTransaction as SourceTransaction
from stripe.api_resources.subscription import Subscription as Subscription
from stripe.api_resources.subscription_item import SubscriptionItem as SubscriptionItem
from stripe.api_resources.subscription_schedule import SubscriptionSchedule as SubscriptionSchedule
from stripe.api_resources.tax_code import TaxCode as TaxCode
from stripe.api_resources.tax_id import TaxId as TaxId
from stripe.api_resources.tax_rate import TaxRate as TaxRate
from stripe.api_resources.three_d_secure import ThreeDSecure as ThreeDSecure
from stripe.api_resources.token import Token as Token
from stripe.api_resources.topup import Topup as Topup
from stripe.api_resources.transfer import Transfer as Transfer
from stripe.api_resources.usage_record import UsageRecord as UsageRecord
from stripe.api_resources.usage_record_summary import UsageRecordSummary as UsageRecordSummary
from stripe.api_resources.webhook_endpoint import WebhookEndpoint as WebhookEndpoint



from _typeshed import SupportsWrite
from typing import TypeVar, overload

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

__version__: str
__all__ = ["lex", "format", "highlight"]

def lex(code, lexer): ...
@overload
def format(tokens, formatter: Formatter[_T], outfile: SupportsWrite[_T]) -> None: ...
@overload
def format(tokens, formatter: Formatter[_T], outfile: None = ...) -> _T: ...
@overload
def highlight(code, lexer, formatter: Formatter[_T], outfile: SupportsWrite[_T]) -> None: ...
@overload
def highlight(code, lexer, formatter: Formatter[_T], outfile: None = ...) -> _T: ...



from typing import Dict, Generic, Iterable, Tuple, TypeVar
from typing_extensions import assert_type

# These do follow `__init__` overloads order:
# mypy and pyright have different opinions about this one:
# mypy raises: 'Need type annotation for "bad"'
# pyright is fine with it.
# bad = dict()
good: Dict[str, str] = dict()
assert_type(good, Dict[str, str])

assert_type(dict(arg=1), Dict[str, int])

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")


class KeysAndGetItem(Generic[_KT, _VT]):
    data: Dict[_KT, _VT]

    def keys(self) -> Iterable[_KT]:
        return self.data.keys()

    def __getitem__(self, __k: _KT) -> _VT:
        return self.data[__k]


kt1: KeysAndGetItem[int, str] = KeysAndGetItem()
assert_type(dict(kt1), Dict[int, str])
dict(kt1, arg="a")  # type: ignore

kt2: KeysAndGetItem[str, int] = KeysAndGetItem()
assert_type(dict(kt2, arg=1), Dict[str, int])


def test_iterable_tuple_overload(x: Iterable[Tuple[int, str]]) -> Dict[int, str]:
    return dict(x)


i1: Iterable[Tuple[int, str]] = [(1, "a"), (2, "b")]
test_iterable_tuple_overload(i1)
dict(i1, arg="a")  # type: ignore

i2: Iterable[Tuple[str, int]] = [("a", 1), ("b", 2)]
assert_type(dict(i2, arg=1), Dict[str, int])

i3: Iterable[str] = ["a.b"]
assert_type(dict(string.split(".") for string in i3), Dict[str, str])
dict(["foo", "bar", "baz"])  # type: ignore



from _typeshed import Incomplete
from collections.abc import Iterator, Mapping as DictMixin

class LazyDict(DictMixin[str, Incomplete]):
    data: dict[str, Incomplete] | None
    def __getitem__(self, key: str) -> Incomplete: ...
    def __contains__(self, key: object) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

class LazyList(list[Incomplete]):
    # does not return `Self` type:
    def __new__(cls, fill_iter: Incomplete | None = ...) -> LazyList: ...  # noqa: Y034

class LazySet(set[Incomplete]):
    # does not return `Self` type:
    def __new__(cls, fill_iter: Incomplete | None = ...) -> LazySet: ...  # noqa: Y034



from typing import Any

from openpyxl.descriptors import Strict
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.styleable import StyleableObject
from openpyxl.utils.bound_dictionary import BoundDictionary

class Dimension(Strict, StyleableObject):
    __fields__: Any
    index: Any
    hidden: Any
    outlineLevel: Any
    outline_level: Any
    collapsed: Any
    style: Any
    def __init__(
        self, index, hidden, outlineLevel, collapsed, worksheet, visible: bool = ..., style: Any | None = ...
    ) -> None: ...
    def __iter__(self): ...
    def __copy__(self): ...

class RowDimension(Dimension):
    __fields__: Any
    r: Any
    s: Any
    ht: Any
    height: Any
    thickBot: Any
    thickTop: Any
    def __init__(
        self,
        worksheet,
        index: int = ...,
        ht: Any | None = ...,
        customHeight: Any | None = ...,
        s: Any | None = ...,
        customFormat: Any | None = ...,
        hidden: bool = ...,
        outlineLevel: int = ...,
        outline_level: Any | None = ...,
        collapsed: bool = ...,
        visible: Any | None = ...,
        height: Any | None = ...,
        r: Any | None = ...,
        spans: Any | None = ...,
        thickBot: Any | None = ...,
        thickTop: Any | None = ...,
        **kw,
    ) -> None: ...
    @property
    def customFormat(self): ...
    @property
    def customHeight(self): ...

class ColumnDimension(Dimension):
    width: Any
    bestFit: Any
    auto_size: Any
    index: Any
    min: Any
    max: Any
    collapsed: Any
    __fields__: Any
    def __init__(
        self,
        worksheet,
        index: str = ...,
        width=...,
        bestFit: bool = ...,
        hidden: bool = ...,
        outlineLevel: int = ...,
        outline_level: Any | None = ...,
        collapsed: bool = ...,
        style: Any | None = ...,
        min: Any | None = ...,
        max: Any | None = ...,
        customWidth: bool = ...,
        visible: Any | None = ...,
        auto_size: Any | None = ...,
    ) -> None: ...
    @property
    def customWidth(self): ...
    def reindex(self) -> None: ...
    def to_tree(self): ...

class DimensionHolder(BoundDictionary):
    worksheet: Any
    max_outline: Any
    default_factory: Any
    def __init__(self, worksheet, reference: str = ..., default_factory: Any | None = ...) -> None: ...
    def group(self, start, end: Any | None = ..., outline_level: int = ..., hidden: bool = ...) -> None: ...
    def to_tree(self): ...

class SheetFormatProperties(Serialisable):
    tagname: str
    baseColWidth: Any
    defaultColWidth: Any
    defaultRowHeight: Any
    customHeight: Any
    zeroHeight: Any
    thickTop: Any
    thickBottom: Any
    outlineLevelRow: Any
    outlineLevelCol: Any
    def __init__(
        self,
        baseColWidth: int = ...,
        defaultColWidth: Any | None = ...,
        defaultRowHeight: int = ...,
        customHeight: Any | None = ...,
        zeroHeight: Any | None = ...,
        thickTop: Any | None = ...,
        thickBottom: Any | None = ...,
        outlineLevelRow: Any | None = ...,
        outlineLevelCol: Any | None = ...,
    ) -> None: ...

class SheetDimension(Serialisable):
    tagname: str
    ref: Any
    def __init__(self, ref: Any | None = ...) -> None: ...
    @property
    def boundaries(self): ...



import builtins

class MySQLError(Exception): ...
class Warning(builtins.Warning, MySQLError): ...
class Error(MySQLError): ...
class InterfaceError(Error): ...
class DatabaseError(Error): ...
class DataError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...
